@@ -25,6 +25,7 @@ extern NSString *HBTitleChangedNotification;
 extern NSString *keyTitleTag;
 
 @class HBOutputPanelController;
+@class DockTextField;
 
 /* We subclass NSView so that our drags show both the icon as well as PresetName columns */
 @interface HBPresetsOutlineView : NSOutlineView
@@ -58,6 +59,29 @@ BOOL                        fIsDragging;
     /* Picture Preview */
     PreviewController            * fPreviewController;
     
+    /* x264 Presets Box */
+    NSArray                      * fX264PresetNames;
+    NSUInteger                     fX264MediumPresetIndex;
+    IBOutlet NSButton            * fX264UseAdvancedOptionsCheck;
+    IBOutlet NSBox               * fX264PresetsBox;
+    IBOutlet NSSlider            * fX264PresetsSlider;
+    IBOutlet NSTextField         * fX264PresetSliderLabel;
+    IBOutlet NSTextField         * fX264PresetSelectedTextField;
+    IBOutlet NSPopUpButton       * fX264TunePopUp;
+    IBOutlet NSTextField         * fX264TunePopUpLabel;
+    IBOutlet NSPopUpButton       * fX264ProfilePopUp;
+    IBOutlet NSTextField         * fX264ProfilePopUpLabel;
+    IBOutlet NSPopUpButton       * fX264LevelPopUp;
+    IBOutlet NSTextField         * fX264LevelPopUpLabel;
+    IBOutlet NSButton            * fX264FastDecodeCheck;
+    IBOutlet NSTextField         * fDisplayX264PresetsAdditonalOptionsTextField;
+    IBOutlet NSTextField         * fDisplayX264PresetsAdditonalOptionsLabel;
+    // Text Field to show the expanded opts from unparse()
+    IBOutlet NSTextField         * fDisplayX264PresetsUnparseTextField;
+    char                         * fX264PresetsUnparsedUTF8String;
+    NSUInteger                     fX264PresetsHeightForUnparse;
+    NSUInteger                     fX264PresetsWidthForUnparse;
+    
     /* Advanced options tab */
     HBAdvancedController         * fAdvancedOptions;
 	IBOutlet NSBox               * fAdvancedView;
@@ -69,10 +93,12 @@ BOOL                        fIsDragging;
     IBOutlet NSTextField         * fQueueStatus;
     
     /* Output panel */
-    HBOutputPanelController       *outputPanel;
+    HBOutputPanelController      * outputPanel;
 	
     /* Source box */
 	IBOutlet NSProgressIndicator * fScanIndicator;
+	IBOutlet NSBox               * fScanHorizontalLine;
+    
 	NSString                     * sourceDisplayName;
     IBOutlet NSTextField         * fSrcDVD2Field;
     IBOutlet NSTextField         * fSrcTitleField;
@@ -140,12 +166,9 @@ BOOL                        fIsDragging;
     IBOutlet NSButton            * fVidTwoPassCheck;
     IBOutlet NSButton            * fVidTurboPassCheck;
 	
-	/* Status read out fileds for picture sizing */
-    IBOutlet NSTextField         * fPictureSizeField;
-    IBOutlet NSTextField         * fPictureCroppingField;
-	
-    /* Status read out fileds for video filters */
-    IBOutlet NSTextField         * fVideoFiltersField;
+    /* Status read out fields for picture settings and video filters */
+    IBOutlet NSTextField         * fPictureSettingsField;
+    IBOutlet NSTextField         * fPictureFiltersField;
 	
 	/* Picture variables */
 	int                        PicOrigOutputWidth;
@@ -174,7 +197,7 @@ BOOL                        fIsDragging;
     IBOutlet NSButton            * fAudioAllowDTSHDPassCheck;
     IBOutlet NSButton            * fAudioAllowDTSPassCheck;
     IBOutlet NSButton            * fAudioAllowMP3PassCheck;
-    IBOutlet NSButton            * fAudioFallbackPopUp;
+    IBOutlet NSPopUpButton       * fAudioFallbackPopUp;
     
     	    
     /* Chapters box */
@@ -256,7 +279,13 @@ BOOL                        fIsDragging;
 	NSString                      * currentSource;
     NSString                      * browsedSourceDisplayName;
     
-    double                         dockIconProgress;
+    /* Dock progress variables */
+    double                          dockIconProgress;
+    
+    BOOL                            fWillScan;
+    NSDockTile                    * dockTile;
+    DockTextField                 * percentField;
+    DockTextField                 * timeField;
 }
 - (int) getPidnum;
 - (IBAction) showAboutPanel:(id)sender;
@@ -273,8 +302,13 @@ BOOL                        fIsDragging;
 
 - (IBAction) cancelScanning:(id)sender;
 
-- (void)     updateUI: (NSTimer *) timer;
-- (void)     enableUI: (bool) enable;
+- (void)     updateUI:                                 (NSTimer*) timer;
+- (void)     enableUI:                                 (bool)     enable;
+- (void)     setupX264PresetsWidgets:                  (id)       sender;
+- (void)     enableX264Widgets:                        (bool)     enable;
+- (IBAction) updateX264Widgets:                        (id)       sender;
+- (IBAction) x264PresetsChangedDisplayExpandedOptions: (id)       sender;
+
 - (IBAction) encodeStartStopPopUpChanged: (id) sender;
 
 
@@ -310,6 +344,11 @@ BOOL                        fIsDragging;
 - (IBAction) calculatePictureSizing: (id) sender;
 - (IBAction) openMainWindow: (id) sender;
 
+/* Text summaries of various settings */
+- (NSString*) pictureSettingsSummary;
+- (NSString*) pictureFiltersSummary;
+- (NSString*) muxerOptionsSummary;
+
 /* Add All titles to the queue */
 - (IBAction) addAllTitlesToQueue: (id) sender;
 - (void) addAllTitlesToQueueAlertDone: (NSWindow *) sheet
@@ -441,5 +480,27 @@ BOOL                        fIsDragging;
 + (unsigned int) maximumNumberOfAllowedAudioTracks;
 - (IBAction) addAllAudioTracks: (id) sender;
 
+// Drag & Drop methods
+- (void)openFiles:(NSArray*)filenames;
+- (void)application:(NSApplication *)sender openFiles:(NSArray *)filenames;
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender;
+- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender;
+
+- (void) updateDockIcon:(double)progress withETA:(NSString*)etaStr;
+
+// x264 system methods
+- (NSString*) x264Preset;
+- (NSString*) x264Tune;
+- (NSString*) x264OptionExtra;
+- (NSString*) h264Profile;
+- (NSString*) h264Level;
+- (void)      setX264Preset:            (NSString*) x264Preset;
+- (void)      setX264Tune:              (NSString*) x264Tune;
+- (void)      setX264OptionExtra:       (NSString*) x264OptionExtra;
+- (void)      setH264Profile:           (NSString*) h264Profile;
+- (void)      setH264Level:             (NSString*) h264Level;
+- (IBAction)  x264PresetsSliderChanged: (id)        sender;
+
+
 @end
 
