@@ -70,28 +70,26 @@ static NSMutableArray *masterBitRateArray = nil;
 {
     if ([HBAudio class] == self)
     {
-        int i, audioMustMatch, shouldAdd;
+        int i, audioMustMatch;
         BOOL muxMKV, muxMP4;
         NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
         NSDictionary *dict;
 
         masterCodecArray = [[NSMutableArray alloc] init]; // knowingly leaked
-        for( i = 0; i < hb_audio_encoders_count; i++ )
+        for (i = 0; i < hb_audio_encoders_count; i++)
         {
-            if( ( hb_audio_encoders[i].encoder & HB_ACODEC_PASS_FLAG ) &&
-                ( hb_audio_encoders[i].encoder != HB_ACODEC_AUTO_PASS ) )
+            if ((hb_audio_encoders[i].encoder & HB_ACODEC_PASS_FLAG) &&
+                (hb_audio_encoders[i].encoder != HB_ACODEC_AUTO_PASS))
             {
-                audioMustMatch = ( hb_audio_encoders[i].encoder & ~HB_ACODEC_PASS_FLAG );
+                audioMustMatch = (hb_audio_encoders[i].encoder & ~HB_ACODEC_PASS_FLAG);
             }
             else
             {
                 audioMustMatch = 0;
             }
-            // don't add HE-AAC encoder if it's unavailable
-            shouldAdd = ( hb_audio_encoders[i].encoder != HB_ACODEC_CA_HAAC ) || encca_haac_available();
-            muxMKV = ( hb_audio_encoders[i].muxers & HB_MUX_MKV ) ? YES : NO;
-            muxMP4 = ( hb_audio_encoders[i].muxers & HB_MUX_MP4 ) ? YES : NO;
-            if( shouldAdd && audioMustMatch )
+            muxMKV = (hb_audio_encoders[i].muxers & HB_MUX_MKV) ? YES : NO;
+            muxMP4 = (hb_audio_encoders[i].muxers & HB_MUX_MP4) ? YES : NO;
+            if (audioMustMatch)
             {
                 [masterCodecArray addObject: [NSDictionary dictionaryWithObjectsAndKeys:
                                               [NSString stringWithUTF8String: hb_audio_encoders[i].human_readable_name], keyAudioCodecName,
@@ -101,7 +99,7 @@ static NSMutableArray *masterBitRateArray = nil;
                                               [NSNumber numberWithInt: audioMustMatch], keyAudioMustMatchTrack,
                                               nil]];
             }
-            else if( shouldAdd )
+            else
             {
                 [masterCodecArray addObject: [NSDictionary dictionaryWithObjectsAndKeys:
                                               [NSString stringWithUTF8String: hb_audio_encoders[i].human_readable_name], keyAudioCodecName,
@@ -226,37 +224,19 @@ static NSMutableArray *masterBitRateArray = nil;
 {
     NSMutableArray *permittedMixdowns = [NSMutableArray array];
     NSDictionary *dict;
-    BOOL shouldAdd;
     int currentMixdown;
 
+    unsigned long long channelLayout = [[track objectForKey: keyAudioInputChannelLayout] unsignedLongLongValue];
     unsigned int count = [masterMixdownArray count];
     int codecCodec = [[codec objectForKey: keyAudioCodec] intValue];
-    int channelLayout = [[track objectForKey: keyAudioInputChannelLayout] intValue];
     int theDefaultMixdown = hb_get_default_mixdown(codecCodec, channelLayout);
-    int theBestMixdown = hb_get_best_mixdown(codecCodec, channelLayout, 0);
 
     for (unsigned int i = 0; i < count; i++)
     {
         dict = [masterMixdownArray objectAtIndex: i];
         currentMixdown = [[dict objectForKey: keyAudioMixdown] intValue];
 
-        // Basically with the way the mixdowns are stored, the assumption from the libhb point of view
-        // currently is that all mixdowns from the best down to mono are supported.
-        if ((currentMixdown != HB_AMIXDOWN_NONE) && (currentMixdown <= theBestMixdown))
-        {
-            shouldAdd = YES;
-        }
-        else if ((currentMixdown == HB_AMIXDOWN_NONE) && (codecCodec & HB_ACODEC_PASS_FLAG))
-        {
-            // "None" mixdown (passthru)
-            shouldAdd = YES;
-        }
-        else
-        {
-            shouldAdd = NO;
-        }
-
-        if (shouldAdd)
+        if (hb_mixdown_is_supported(currentMixdown, codecCodec, channelLayout))
         {
             [permittedMixdowns addObject: dict];
         }
