@@ -90,9 +90,9 @@ NSString *HBMixdownChangedNotification = @"HBMixdownChangedNotification";
         if ([anAudio enabled])
         {
             NSString *prefix = [NSString stringWithFormat: @"Audio%d", counter + 1];
-            NSNumber *sampleRateToUse = (0 == [[[anAudio sampleRate] objectForKey: keyAudioSamplerate] intValue]) ?
-            [[anAudio track] objectForKey: keyAudioInputSampleRate] :
-            [[anAudio sampleRate] objectForKey: keyAudioSamplerate];
+            NSNumber *sampleRateToUse = ([[[anAudio sampleRate] objectForKey: keyAudioSamplerate] intValue] == 0 ?
+                                         [[anAudio track] objectForKey: keyAudioInputSampleRate] :
+                                         [[anAudio sampleRate] objectForKey: keyAudioSamplerate]);
 
             [aDict setObject: [[anAudio track] objectForKey: keyAudioTrackIndex] forKey: [prefix stringByAppendingString: @"Track"]];
             [aDict setObject: [[anAudio track] objectForKey: keyAudioTrackName] forKey: [prefix stringByAppendingString: @"TrackDescription"]];
@@ -150,25 +150,27 @@ NSString *HBMixdownChangedNotification = @"HBMixdownChangedNotification";
     unsigned int audioArrayCount = [self countOfAudioArray];
     for (i = 0; i < audioArrayCount; i++)
     {
-        HBAudio *anAudio = [self objectInAudioArrayAtIndex: i];
+        HBAudio *anAudio = [self objectInAudioArrayAtIndex:i];
         if ([anAudio enabled])
         {
-            NSNumber *sampleRateToUse = (0 == [[[anAudio sampleRate] objectForKey: keyAudioSamplerate] intValue]) ?
-            [[anAudio track] objectForKey: keyAudioInputSampleRate] :
-            [[anAudio sampleRate] objectForKey: keyAudioSamplerate];
+            NSNumber *sampleRateToUse = ([[[anAudio sampleRate] objectForKey:keyAudioSamplerate] intValue] == 0 ?
+                                         [[anAudio track]       objectForKey:keyAudioInputSampleRate] :
+                                         [[anAudio sampleRate]  objectForKey:keyAudioSamplerate]);
 
-            hb_audio_config_t *audio = (hb_audio_config_t *) calloc(1, sizeof(*audio));
+            hb_audio_config_t *audio = (hb_audio_config_t*)calloc(1, sizeof(*audio));
             hb_audio_config_init(audio);
-            audio->in.track = [[[anAudio track] objectForKey: keyAudioTrackIndex] intValue] - 1;
+            audio->in.track          = [[[anAudio track] objectForKey:keyAudioTrackIndex] intValue] - 1;
             /* We go ahead and assign values to our audio->out.<properties> */
-            audio->out.track = audio->in.track;
-            audio->out.codec = [[[anAudio codec] objectForKey: keyAudioCodec] intValue];
-            audio->out.compression_level = hb_get_default_audio_compression(audio->out.codec);
-            audio->out.mixdown = [[[anAudio mixdown] objectForKey: keyAudioMixdown] intValue];
-            audio->out.bitrate = [[[anAudio bitRate] objectForKey: keyAudioBitrate] intValue];
-            audio->out.samplerate = [sampleRateToUse intValue];
-            audio->out.dynamic_range_compression = [[anAudio drc] floatValue];
-            audio->out.gain = [[anAudio gain] floatValue];
+            audio->out.track                     = audio->in.track;
+            audio->out.codec                     = [[[anAudio codec]   objectForKey:keyAudioCodec]   intValue];
+            audio->out.compression_level         = hb_get_default_audio_compression(audio->out.codec);
+            audio->out.mixdown                   = [[[anAudio mixdown] objectForKey:keyAudioMixdown] intValue];
+            audio->out.normalize_mix_level       = 0;
+            audio->out.bitrate                   = [[[anAudio bitRate] objectForKey:keyAudioBitrate] intValue];
+            audio->out.samplerate                = [sampleRateToUse  intValue];
+            audio->out.dynamic_range_compression = [[anAudio drc]  floatValue];
+            audio->out.gain                      = [[anAudio gain] floatValue];
+            audio->out.dither_method             = hb_audio_dither_get_default();
 
             hb_audio_add(aJob, audio);
             free(audio);
@@ -345,7 +347,7 @@ NSString *HBMixdownChangedNotification = @"HBMixdownChangedNotification";
                 [dict setObject:[NSNumber numberWithFloat:0.0] forKey:@"AudioTrackGainSlider"];
             }
 
-            // map legacy passthru mixdowns
+            // map legacy mixdowns
             key = [dict objectForKey: @"AudioMixdown"];
             if ([key isEqualToString: @"AC3 Passthru"] ||
                 [key isEqualToString: @"DTS Passthru"] ||
@@ -353,6 +355,10 @@ NSString *HBMixdownChangedNotification = @"HBMixdownChangedNotification";
             {
                 [dict setObject: @"None" forKey: @"AudioMixdown"];
             }
+            else if ([key isEqualToString: @"6-channel discrete"])
+            {
+                [dict setObject: @"5.1 Channels" forKey: @"AudioMixdown"];
+            }
 
             // If our preset wants us to support a codec that the track does not support, instead
             // of changing the codec we remove the audio instead.
@@ -426,24 +432,51 @@ NSString *HBMixdownChangedNotification = @"HBMixdownChangedNotification";
 
 {
     id whatToUse = [self _presetAudioArrayFromPreset: aPreset];
-    NSString *preferredLanguageName = [[NSUserDefaults standardUserDefaults] stringForKey: @"DefaultLanguage"];
-    int preferredLanguage = [self _trackWithTitlePrefix: preferredLanguageName defaultIfNotFound: 1];
+    NSMutableArray *tracksToAdd = [[NSMutableArray alloc] init];
 
-    // Reinitialize the configured list of audio tracks
-    [self _clearAudioArray];
+    NSArray* preferredLanguages = [NSArray arrayWithObjects: 
+                          [[NSUserDefaults standardUserDefaults] stringForKey: @"DefaultLanguage"],
+                          [[NSUserDefaults standardUserDefaults] stringForKey: @"AlternateLanguage"],
+                          nil];
 
-    [self _processPresetAudioArray: whatToUse forTrack: preferredLanguage andType: [[aPreset objectForKey: @"Type"] intValue]];
+    // Add tracks of Default and Alternate Language by name
+    for(id languageName in preferredLanguages)
+    {
+        int trackNumber = [self _trackWithTitlePrefix: languageName defaultIfNotFound: 0];
+        
+        if(trackNumber > 0 && [tracksToAdd indexOfObject:[NSNumber numberWithInt:trackNumber]] == NSNotFound)
+        {
+            [tracksToAdd addObject:[NSNumber numberWithInt:trackNumber]];
+        }
+    }
+
+    // If no preferred Language was found, add standard track 1
+    if([tracksToAdd count] == 0)
+    {
+        [tracksToAdd addObject:[NSNumber numberWithInt:1]];
+    }
+
+    // If all tracks should be added, add all track numbers that are not yet processed
     if (allTracks)
     {
         unsigned int count = [masterTrackArray count];
         for (unsigned int i = 1; i < count; i++)
         {
-            if (i != preferredLanguage)
+            NSNumber *trackNumber = [NSNumber numberWithInt:i];
+            if([tracksToAdd indexOfObject:trackNumber] == NSNotFound)
             {
-                [self _processPresetAudioArray: whatToUse forTrack: i andType: [[aPreset objectForKey: @"Type"] intValue]];
+               [tracksToAdd addObject:trackNumber];
             }
         }
     }
+         
+    // Reinitialize the configured list of audio tracks
+    [self _clearAudioArray];
+
+    for(id trackNumber in tracksToAdd)
+    {
+        [self _processPresetAudioArray: whatToUse forTrack:[trackNumber intValue] andType: [[aPreset objectForKey: @"Type"] intValue]];
+    }
 }
 
 - (void) _ensureAtLeastOneNonEmptyTrackExists
@@ -598,7 +631,7 @@ NSString *HBMixdownChangedNotification = @"HBMixdownChangedNotification";
                                        [NSNumber numberWithInt: audio->in.bitrate / 1000], keyAudioInputBitrate,
                                        [NSNumber numberWithInt: audio->in.samplerate], keyAudioInputSampleRate,
                                        [NSNumber numberWithInt: audio->in.codec], keyAudioInputCodec,
-                                       [NSNumber numberWithInt: audio->in.channel_layout], keyAudioInputChannelLayout,
+                                       [NSNumber numberWithUnsignedLongLong: audio->in.channel_layout], keyAudioInputChannelLayout,
                                        nil]];
         }
         self.masterTrackArray = newTrackArray;
