@@ -340,7 +340,7 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
 
         // Set up a reasonable tooltip, and image
         [toolbarItem setToolTip: @"Start Encoding"];
-        [toolbarItem setImage: [NSImage imageNamed: @"Play"]];
+        [toolbarItem setImage: [NSImage imageNamed: @"encode"]];
 
         // Tell the item what message to send when it is clicked
         [toolbarItem setTarget: self];
@@ -357,7 +357,7 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
 
         // Set up a reasonable tooltip, and image
         [toolbarItem setToolTip: @"Pause Encoding"];
-        [toolbarItem setImage: [NSImage imageNamed: @"Pause"]];
+        [toolbarItem setImage: [NSImage imageNamed: @"pauseencode"]];
 
         // Tell the item what message to send when it is clicked
         [toolbarItem setTarget: self];
@@ -420,7 +420,7 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
         if ((s.state == HB_STATE_PAUSED) || (s.state == HB_STATE_WORKING) || (s.state == HB_STATE_MUXING))
         {
             enable = YES;
-            [toolbarItem setImage:[NSImage imageNamed: @"Stop"]];
+            [toolbarItem setImage:[NSImage imageNamed: @"stopencode"]];
             [toolbarItem setLabel: @"Stop"];
             [toolbarItem setToolTip: @"Stop Encoding"];
         }
@@ -428,7 +428,7 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
         else if (fPendingCount > 0)
         {
             enable = YES;
-            [toolbarItem setImage:[NSImage imageNamed: @"Play"]];
+            [toolbarItem setImage:[NSImage imageNamed: @"encode"]];
             [toolbarItem setLabel: @"Start"];
             [toolbarItem setToolTip: @"Start Encoding"];
         }
@@ -436,7 +436,7 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
         else
         {
             enable = NO;
-            [toolbarItem setImage:[NSImage imageNamed: @"Play"]];
+            [toolbarItem setImage:[NSImage imageNamed: @"encode"]];
             [toolbarItem setLabel: @"Start"];
             [toolbarItem setToolTip: @"Start Encoding"];
         }
@@ -447,7 +447,7 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
         if (s.state == HB_STATE_PAUSED)
         {
             enable = YES;
-            [toolbarItem setImage:[NSImage imageNamed: @"Play"]];
+            [toolbarItem setImage:[NSImage imageNamed: @"encode"]];
             [toolbarItem setLabel: @"Resume"];
             [toolbarItem setToolTip: @"Resume Encoding"];
        }
@@ -455,14 +455,14 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
         else if ((s.state == HB_STATE_WORKING) || (s.state == HB_STATE_MUXING))
         {
             enable = YES;
-            [toolbarItem setImage:[NSImage imageNamed: @"Pause"]];
+            [toolbarItem setImage:[NSImage imageNamed: @"pauseencode"]];
             [toolbarItem setLabel: @"Pause"];
             [toolbarItem setToolTip: @"Pause Encoding"];
         }
         else
         {
             enable = NO;
-            [toolbarItem setImage:[NSImage imageNamed: @"Pause"]];
+            [toolbarItem setImage:[NSImage imageNamed: @"pauseencode"]];
             [toolbarItem setLabel: @"Pause"];
             [toolbarItem setToolTip: @"Pause Encoding"];
         }
@@ -839,66 +839,26 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         int itemRowNum = [outlineView rowForItem: item];
         NSMutableDictionary *queueItemToCheck = [outlineView itemAtRow: itemRowNum];
         
-        /* Check to see if we need to allow for mp4 opts */
-        BOOL mp4OptsPresent = NO;
-        if ([[queueItemToCheck objectForKey:@"FileFormat"] isEqualToString: @"MP4 file"])
+        /* Check to see if we need to allow for container options */
+        if ([[queueItemToCheck objectForKey:@"MuxerOptionsSummary"] length])
         {
-            
-            if( [[queueItemToCheck objectForKey:@"Mp4LargeFile"] intValue] == 1)
-            {
-                mp4OptsPresent = YES;
-            }
-            if( [[queueItemToCheck objectForKey:@"Mp4HttpOptimize"] intValue] == 1)
-            {
-                mp4OptsPresent = YES;
-            }
-            if( [[queueItemToCheck objectForKey:@"Mp4iPodCompatible"] intValue] == 1)
-            {
-                mp4OptsPresent = YES;
-            }
-        }
-        
-        if (mp4OptsPresent == YES)
-        {
-            itemHeightForDisplay +=  rowHeightNonTitle;   
+            itemHeightForDisplay += rowHeightNonTitle;
         }
         
         /* check to see if we need to allow for the Picture Filters row */
-        BOOL pictureFiltersPresent = NO;
-        if( [[queueItemToCheck objectForKey:@"PictureDetelecine"] intValue] > 0)
+        if ([[queueItemToCheck objectForKey:@"PictureFiltersSummary"] length])
         {
-            pictureFiltersPresent = YES;
-        }
-        if( [[queueItemToCheck objectForKey:@"PictureDecomb"] intValue] > 0)
-        {
-            pictureFiltersPresent = YES;
-        }
-        if( [[queueItemToCheck objectForKey:@"PictureDeinterlace"] intValue] > 0)
-        {
-            pictureFiltersPresent = YES;
-        }
-        if( [[queueItemToCheck objectForKey:@"PictureDenoise"] intValue] > 0)
-        {
-            pictureFiltersPresent = YES;
-        }
-        if( [[queueItemToCheck objectForKey:@"PictureDeblock"] intValue] > 0)
-        {
-            pictureFiltersPresent = YES;
-        }
-        if( [[queueItemToCheck objectForKey:@"VideoGrayScale"] intValue] > 0)
-        {
-            pictureFiltersPresent = YES;
+            itemHeightForDisplay += rowHeightNonTitle;
         }
         
-        if (pictureFiltersPresent == YES)
-        {
-            itemHeightForDisplay +=  rowHeightNonTitle;
-        }
-        
-        /* check to see if we need a line to display x264 options */
+        /* check to see if we need a line to display x264/lavc options */
         if ([[queueItemToCheck objectForKey:@"VideoEncoder"] isEqualToString: @"H.264 (x264)"])
         {
-            itemHeightForDisplay +=  rowHeightNonTitle;
+            itemHeightForDisplay += rowHeightNonTitle * 2;
+        }
+        else if (![[queueItemToCheck objectForKey:@"VideoEncoder"] isEqualToString: @"VP3 (Theora)"])
+        {
+            itemHeightForDisplay += rowHeightNonTitle;
         }
         
         /* check to see how many audio track lines to allow for */
@@ -915,7 +875,7 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
                 autoPassthruPresent = YES;
             }
 		}
-		itemHeightForDisplay += (actualCountOfAudioTracks * rowHeightNonTitle);
+		itemHeightForDisplay += (actualCountOfAudioTracks * rowHeightNonTitle * 2);
         
         if (autoPassthruPresent == YES)
         {
@@ -1113,33 +1073,14 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         [finalString appendString: @"Format: " withAttributes:detailBoldAttr];
         [finalString appendString: jobFormatInfo withAttributes:detailAttr];
         
-        /* Optional String for mp4 options */
-        if ([[item objectForKey:@"FileFormat"] isEqualToString: @"MP4 file"])
+        /* Optional String for muxer options */
+        if ([[item objectForKey:@"MuxerOptionsSummary"] length])
         {
-            NSString * MP4Opts = @"";
-            BOOL mp4OptsPresent = NO;
-            if( [[item objectForKey:@"Mp4LargeFile"] intValue] == 1)
-            {
-                mp4OptsPresent = YES;
-                MP4Opts = [MP4Opts stringByAppendingString:@" - Large file size"];
-            }
-            if( [[item objectForKey:@"Mp4HttpOptimize"] intValue] == 1)
-            {
-                mp4OptsPresent = YES;
-                MP4Opts = [MP4Opts stringByAppendingString:@" - Web optimized"];
-            }
-            
-            if( [[item objectForKey:@"Mp4iPodCompatible"] intValue] == 1)
-            {
-                mp4OptsPresent = YES;
-                MP4Opts = [MP4Opts stringByAppendingString:@" - iPod 5G support "];
-            }
-            if (mp4OptsPresent == YES)
-            {
-                [finalString appendString: @"MP4 Options: " withAttributes:detailBoldAttr];
-                [finalString appendString: MP4Opts withAttributes:detailAttr];
-                [finalString appendString:@"\n" withAttributes:detailAttr];
-            }
+            NSString *containerOptions = [NSString stringWithFormat:@"%@",
+                                          [item objectForKey:@"MuxerOptionsSummary"]];
+            [finalString appendString:@"Container Options: " withAttributes:detailBoldAttr];
+            [finalString appendString:containerOptions       withAttributes:detailAttr];
+            [finalString appendString:@"\n"                  withAttributes:detailAttr];
         }
         
         /* Fourth Line (Destination Path)*/
@@ -1148,121 +1089,24 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         [finalString appendString:@"\n" withAttributes:detailAttr];
         
         /* Fifth Line Picture Details*/
-        NSString * pictureInfo;
-        pictureInfo = [NSString stringWithFormat:@"%@", [item objectForKey:@"PictureSizingSummary"]];
+        NSString *pictureInfo = [NSString stringWithFormat:@"%@",
+                                 [item objectForKey:@"PictureSettingsSummary"]];
         if ([[item objectForKey:@"PictureKeepRatio"] intValue] == 1)
         {
             pictureInfo = [pictureInfo stringByAppendingString:@" Keep Aspect Ratio"];
         }
-        
-        if ([[item objectForKey:@"VideoGrayScale"] intValue] == 1)
-        {
-            pictureInfo = [pictureInfo stringByAppendingString:@", Grayscale"];
-        }
-        
-        [finalString appendString: @"Picture: " withAttributes:detailBoldAttr];
-        [finalString appendString: pictureInfo withAttributes:detailAttr];
-        [finalString appendString:@"\n" withAttributes:detailAttr];
+        [finalString appendString:@"Picture: " withAttributes:detailBoldAttr];
+        [finalString appendString:pictureInfo  withAttributes:detailAttr];
+        [finalString appendString:@"\n"        withAttributes:detailAttr];
         
         /* Optional String for Picture Filters */
-        
-        NSString * pictureFilters = @"";
-        BOOL pictureFiltersPresent = NO;
-        
-        if( [[item objectForKey:@"PictureDetelecine"] intValue] == 1)
+        if ([[item objectForKey:@"PictureFiltersSummary"] length])
         {
-            pictureFiltersPresent = YES;
-            pictureFilters = [pictureFilters stringByAppendingString:[NSString stringWithFormat:@" - Detelecine (%@)",[item objectForKey:@"PictureDetelecineCustom"]]];
-        }
-        else if( [[item objectForKey:@"PictureDetelecine"] intValue] == 2)
-        {
-            pictureFiltersPresent = YES;
-            pictureFilters = [pictureFilters stringByAppendingString:@" - Detelecine (Default)"];
-        }
-        
-        if( [[item objectForKey:@"PictureDecombDeinterlace"] intValue] == 1)
-        {
-            if ([[item objectForKey:@"PictureDecomb"] intValue] != 0)
-            {
-                if( [[item objectForKey:@"PictureDecomb"] intValue] == 1)
-                {
-                    pictureFiltersPresent = YES;
-                    pictureFilters = [pictureFilters stringByAppendingString:[NSString stringWithFormat:@" - Decomb (%@)",[item objectForKey:@"PictureDecombCustom"]]];
-                }
-                else if( [[item objectForKey:@"PictureDecomb"] intValue] == 2)
-                {
-                    pictureFiltersPresent = YES;
-                    pictureFilters = [pictureFilters stringByAppendingString:@" - Decomb (Default)"];
-                }
-                else if( [[item objectForKey:@"PictureDecomb"] intValue] == 3)
-                {
-                    pictureFiltersPresent = YES;
-                    pictureFilters = [pictureFilters stringByAppendingString:@" - Decomb (Fast)"];
-                }
-            }
-        }
-        else
-        {
-            if ([[item objectForKey:@"PictureDeinterlace"] intValue] != 0)
-            {
-                pictureFiltersPresent = YES;
-                if ([[item objectForKey:@"PictureDeinterlace"] intValue] == 1)
-                {
-                    pictureFilters = [pictureFilters stringByAppendingString:[NSString stringWithFormat:@" - Deinterlace (%@)",[item objectForKey:@"PictureDeinterlaceCustom"]]];            
-                }
-                else if ([[item objectForKey:@"PictureDeinterlace"] intValue] == 2)
-                {
-                    pictureFilters = [pictureFilters stringByAppendingString:@" - Deinterlace (Fast)"];
-                }
-                else if ([[item objectForKey:@"PictureDeinterlace"] intValue] == 3)
-                {
-                    pictureFilters = [pictureFilters stringByAppendingString:@" - Deinterlace (Slow)"];           
-                }
-                else if ([[item objectForKey:@"PictureDeinterlace"] intValue] == 4)
-                {
-                    pictureFilters = [pictureFilters stringByAppendingString:@" - Deinterlace (Slower)"];            
-                }
-                
-            }
-        }
-        if ([[item objectForKey:@"PictureDenoise"] intValue] != 0)
-        {
-            pictureFiltersPresent = YES;
-            if ([[item objectForKey:@"PictureDenoise"] intValue] == 1)
-            {
-                pictureFilters = [pictureFilters stringByAppendingString:[NSString stringWithFormat:@" - Denoise (%@)",[item objectForKey:@"PictureDenoiseCustom"]]];            
-            }
-            else if ([[item objectForKey:@"PictureDenoise"] intValue] == 2)
-            {
-                pictureFilters = [pictureFilters stringByAppendingString:@" - Denoise (Weak)"];
-            }
-            else if ([[item objectForKey:@"PictureDenoise"] intValue] == 3)
-            {
-                pictureFilters = [pictureFilters stringByAppendingString:@" - Denoise (Medium)"];           
-            }
-            else if ([[item objectForKey:@"PictureDenoise"] intValue] == 4)
-            {
-                pictureFilters = [pictureFilters stringByAppendingString:@" - Denoise (Strong)"];            
-            }
-            
-        }
-        if ([[item objectForKey:@"PictureDeblock"] intValue] != 0)
-        {
-            pictureFiltersPresent = YES;
-            pictureFilters = [pictureFilters stringByAppendingString: [NSString stringWithFormat:@" - Deblock (pp7) (%d)",[[item objectForKey:@"PictureDeblock"] intValue]]];
-        }
-        
-        if ([[item objectForKey:@"VideoGrayScale"] intValue] == 1)
-        {
-            pictureFiltersPresent = YES;
-            pictureFilters = [pictureFilters stringByAppendingString:@" - Grayscale"];
-        }
-        
-        if (pictureFiltersPresent == YES)
-        {
-            [finalString appendString: @"Filters: " withAttributes:detailBoldAttr];
-            [finalString appendString: pictureFilters withAttributes:detailAttr];
-            [finalString appendString:@"\n" withAttributes:detailAttr];
+            NSString *pictureFilters = [NSString stringWithFormat:@"%@",
+                                        [item objectForKey:@"PictureFiltersSummary"]];
+            [finalString appendString:@"Filters: "   withAttributes:detailBoldAttr];
+            [finalString appendString:pictureFilters withAttributes:detailAttr];
+            [finalString appendString:@"\n"          withAttributes:detailAttr];
         }
         
         /* Sixth Line Video Details*/
@@ -1315,29 +1159,61 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         
         if ([[item objectForKey:@"VideoEncoder"] isEqualToString: @"H.264 (x264)"])
         {
-            [finalString appendString: @"x264 Options: " withAttributes:detailBoldAttr];
-            if ([item objectForKey:@"x264Option"])
+            /* we are using x264 */
+            NSString *x264Info = @"";
+            if ([[item objectForKey:@"x264UseAdvancedOptions"] intValue])
             {
-                [finalString appendString: [item objectForKey:@"x264Option"] withAttributes:detailAttr];
+                // we are using the old advanced panel
+                if ([item objectForKey:@"x264Option"] &&
+                    [[item objectForKey:@"x264Option"] length])
+                {
+                    x264Info = [x264Info stringByAppendingString: [item objectForKey:@"x264Option"]];
+                }
+                else
+                {
+                    x264Info = [x264Info stringByAppendingString: @"default settings"];
+                }
             }
             else
             {
-                [finalString appendString: @"x264 defaults" withAttributes:detailAttr];   
+                // we are using the x264 system
+                x264Info = [x264Info stringByAppendingString: [NSString stringWithFormat:@"Preset: %@", [item objectForKey:@"x264Preset"]]];
+                if ([[item objectForKey:@"x264Tune"] length])
+                {
+                    x264Info = [x264Info stringByAppendingString: [NSString stringWithFormat:@" - Tune: %@", [item objectForKey:@"x264Tune"]]];
+                }
+                if ([[item objectForKey:@"x264OptionExtra"] length])
+                {
+                    x264Info = [x264Info stringByAppendingString: [NSString stringWithFormat:@" - Options: %@", [item objectForKey:@"x264OptionExtra"]]];
+                }
+                if ([[item objectForKey:@"h264Profile"] length])
+                {
+                    x264Info = [x264Info stringByAppendingString: [NSString stringWithFormat:@" - Profile: %@", [item objectForKey:@"h264Profile"]]];
+                }
+                if ([[item objectForKey:@"h264Level"] length])
+                {
+                    x264Info = [x264Info stringByAppendingString: [NSString stringWithFormat:@" - Level: %@", [item objectForKey:@"h264Level"]]];
+                }
             }
+            [finalString appendString: @"x264: " withAttributes:detailBoldAttr];
+            [finalString appendString: x264Info withAttributes:detailAttr];
             [finalString appendString:@"\n" withAttributes:detailAttr];
         }
-        /*If we are not x264 and we are not Theora then we must be FFmpeg (lavc) */
         else if (![[item objectForKey:@"VideoEncoder"] isEqualToString: @"VP3 (Theora)"])
         {
-            [finalString appendString: @"FFmpeg (lavc) Options: " withAttributes:detailBoldAttr];
-            if ([item objectForKey:@"lavcOption"])
+            /* we are using libavcodec */
+            NSString *lavcInfo = @"";
+            if ([item objectForKey:@"lavcOption"] &&
+                [[item objectForKey:@"lavcOption"] length])
             {
-                [finalString appendString: [item objectForKey:@"lavcOption"] withAttributes:detailAttr];
+                lavcInfo = [lavcInfo stringByAppendingString: [item objectForKey:@"lavcOption"]];
             }
             else
             {
-                [finalString appendString: @"FFmpeg (lavc) defaults" withAttributes:detailAttr];   
+                lavcInfo = [lavcInfo stringByAppendingString: @"default settings"];
             }
+            [finalString appendString: @"ffmpeg: " withAttributes:detailBoldAttr];
+            [finalString appendString: lavcInfo withAttributes:detailAttr];
             [finalString appendString:@"\n" withAttributes:detailAttr];
         }
         
