@@ -1,16 +1,37 @@
-/*  IUserSettingService.cs $
-    This file is part of the HandBrake source code.
-    Homepage: <http://handbrake.fr/>.
-    It may be used under the terms of the GNU General Public License. */
-
+// --------------------------------------------------------------------------------------------------------------------
+// <copyright file="IUserSettingService.cs" company="HandBrake Project (http://handbrake.fr)">
+//   This file is part of the HandBrake source code - It may be used under the terms of the GNU General Public License.
+// </copyright>
+// <summary>
+//   The User Setting Service Interace.
+// </summary>
+// --------------------------------------------------------------------------------------------------------------------
 
 namespace HandBrake.ApplicationServices.Services.Interfaces
 {
+    using HandBrake.ApplicationServices.EventArgs;
+
+    /// <summary>
+    /// The setting event handler.
+    /// </summary>
+    /// <param name="sender">
+    /// The sender.
+    /// </param>
+    /// <param name="e">
+    /// The e.
+    /// </param>
+    public delegate void SettingEventHandler(object sender, SettingChangedEventArgs e);
+
     /// <summary>
     /// The User Setting Service Interace.
     /// </summary>
     public interface IUserSettingService
     {
+        /// <summary>
+        /// The setting changed.
+        /// </summary>
+        event SettingEventHandler SettingChanged;
+
         /// <summary>
         /// Set the specified user setting.
         /// </summary>
