@@ -1,10 +1,11 @@
-﻿/*  PresetService.cs $
-    This file is part of the HandBrake source code.
-    Homepage: <http://handbrake.fr>.
-    It may be used under the terms of the GNU General Public License. */
-
-using HandBrake.ApplicationServices.Model.Encoding;
-using HandBrake.ApplicationServices.Utilities;
+﻿// --------------------------------------------------------------------------------------------------------------------
+// <copyright file="PresetService.cs" company="HandBrake Project (http://handbrake.fr)">
+//   This file is part of the HandBrake source code - It may be used under the terms of the GNU General Public License.
+// </copyright>
+// <summary>
+//   The preset service manages HandBrake's presets
+// </summary>
+// --------------------------------------------------------------------------------------------------------------------
 
 namespace HandBrake.ApplicationServices.Services
 {
@@ -21,19 +22,14 @@ namespace HandBrake.ApplicationServices.Services
     using HandBrake.ApplicationServices.Exceptions;
     using HandBrake.ApplicationServices.Model;
     using HandBrake.ApplicationServices.Services.Interfaces;
+    using HandBrake.ApplicationServices.Model.Encoding;
+    using HandBrake.ApplicationServices.Utilities;
 
     /// <summary>
     /// The preset service manages HandBrake's presets
     /// </summary>
     public class PresetService : IPresetService
     {
-        /**
-         * TODO:
-         * - Wire this into the Forms and WPF UI's
-         * - Note: This is untested so far. It'll likely need fixes before it can be used.
-         * - Maybe change the collection to a dictionary to allow easier lookups?
-         **/
-
         #region Private Variables
 
         /// <summary>
@@ -44,7 +40,12 @@ namespace HandBrake.ApplicationServices.Services
         /// <summary>
         /// The User Setting Service
         /// </summary>
-        private IUserSettingService userSettingService = ServiceManager.UserSettingService;
+        private readonly IUserSettingService userSettingService;
+
+        /// <summary>
+        /// User Preset Default Catgory Name
+        /// </summary>
+        public static string UserPresetCatgoryName = "User Presets";
 
         /// <summary>
         /// The User Preset file
@@ -63,11 +64,17 @@ namespace HandBrake.ApplicationServices.Services
 
         #endregion
 
+
         /// <summary>
         /// Initializes a new instance of the <see cref="PresetService"/> class.
         /// </summary>
-        public PresetService()
+        /// <param name="userSettingService">
+        /// The user Setting Service.
+        /// </param>
+        public PresetService(IUserSettingService userSettingService)
         {
+            this.userSettingService = userSettingService;
+
             // If the preset file doesn't exist. Create it.
             if (!File.Exists(this.builtInPresetFile))
             {
@@ -107,7 +114,8 @@ namespace HandBrake.ApplicationServices.Services
         #region Public Methods
 
         /// <summary>
-        /// Add a new preset to the system
+        /// Add a new preset to the system.
+        /// Performs an Update if it already exists
         /// </summary>
         /// <param name="preset">
         /// A Preset to add
@@ -127,8 +135,9 @@ namespace HandBrake.ApplicationServices.Services
                 this.UpdatePresetFiles();
                 return true;
             }
-
-            return false;
+            
+            this.Update(preset);
+            return true;
         }
 
         /// <summary>
@@ -144,11 +153,11 @@ namespace HandBrake.ApplicationServices.Services
             {
                 if (preset.Name == update.Name)
                 {
-                    preset.Query = update.Query;
                     preset.Task = update.Task;
-                    preset.AudioPassthruSettings = update.AudioPassthruSettings;
-                    preset.CropSettings = update.CropSettings;
                     preset.UsePictureFilters = update.UsePictureFilters;
+                    preset.PictureSettingsMode = update.PictureSettingsMode;
+                    preset.Category = update.Category;
+                    preset.Description = update.Description;
 
                     // Update the presets file
                     this.UpdatePresetFiles();
@@ -214,7 +223,6 @@ namespace HandBrake.ApplicationServices.Services
             this.UpdatePresetFiles();
         }
 
-
         /// <summary>
         /// Get a Preset
         /// </summary>
@@ -252,9 +260,6 @@ namespace HandBrake.ApplicationServices.Services
         /// <summary>
         /// Reads the CLI's CLI output format and load's them into the preset List Preset
         /// </summary>
-        /// <param name="cliPath">
-        /// The Path to the CLI, leave blank for current folder.
-        /// </param>
         public void UpdateBuiltInPresets()
         {
             // Create a new tempory file and execute the CLI to get the built in Presets.
@@ -297,21 +302,18 @@ namespace HandBrake.ApplicationServices.Services
                             Regex r = new Regex("(:  )"); // Split on hyphens. 
                             string[] presetName = r.Split(line);
 
-                            bool pic = presetName[2].Contains("crop");
-
                             Preset newPreset = new Preset
                                 {
                                     Category = category,
                                     Name = presetName[0].Replace("+", string.Empty).Trim(),
-                                    Query = presetName[2],
-                                    Version = this.userSettingService.GetUserSetting<string>(ASUserSettingConstants.HandBrakeVersion),
-                                    CropSettings = pic,
+                                    Version = VersionHelper.GetVersion(),
                                     Description = string.Empty, // Maybe one day we will populate this.
                                     IsBuildIn = true,
+                                    UsePictureFilters = true,
                                     Task = QueryParserUtility.Parse(presetName[2])
                                 };
 
-                            newPreset.AudioPassthruSettings = new AllowedPassthru(false); // We don't want to override the built-in preset
+                            newPreset.Task.AllowedPassthruOptions = new AllowedPassthru(true); // We don't want to override the built-in preset
                             
                             if (newPreset.Name == "Normal")
                             {
@@ -352,7 +354,7 @@ namespace HandBrake.ApplicationServices.Services
                 List<Preset> preset = this.presets.Where(p => p.IsBuildIn).ToList();
                 if (preset.Count > 0)
                 {
-                    if (preset[0].Version != this.userSettingService.GetUserSetting<string>(ASUserSettingConstants.HandBrakeVersion))
+                    if (preset[0].Version != VersionHelper.GetVersion())
                     {
                         this.UpdateBuiltInPresets();
                         return true;
@@ -432,15 +434,16 @@ namespace HandBrake.ApplicationServices.Services
             {
                 if (File.Exists(this.builtInPresetFile))
                 {
-                    StreamReader reader = new StreamReader(this.builtInPresetFile);
-                    List<Preset> list = (List<Preset>)Ser.Deserialize(reader);
-                    foreach (Preset preset in list)
+                    using (StreamReader reader = new StreamReader(this.builtInPresetFile))
                     {
-                        preset.IsBuildIn = true; // Older versions did not have this flag so explicitly make sure it is set.
-                        this.presets.Add(preset);
+                        List<Preset> list = (List<Preset>)Ser.Deserialize(reader);
+                        foreach (Preset preset in list)
+                        {
+                            preset.IsBuildIn = true;
+                                // Older versions did not have this flag so explicitly make sure it is set.
+                            this.presets.Add(preset);
+                        }
                     }
-
-                    reader.Close();
                 } 
             }
             catch (Exception exc)
@@ -455,14 +458,14 @@ namespace HandBrake.ApplicationServices.Services
             {
                 if (File.Exists(this.userPresetFile))
                 {
-                    StreamReader reader = new StreamReader(this.userPresetFile);
-                    List<Preset> list = (List<Preset>)Ser.Deserialize(reader);
-                    foreach (Preset preset in list)
+                    using (StreamReader reader = new StreamReader(this.userPresetFile))
                     {
-                        this.presets.Add(preset);
+                        List<Preset> list = (List<Preset>)Ser.Deserialize(reader);
+                        foreach (Preset preset in list)
+                        {
+                            this.presets.Add(preset);
+                        }
                     }
-
-                    reader.Close();
                 }
             }
             catch (Exception exc)
@@ -479,18 +482,20 @@ namespace HandBrake.ApplicationServices.Services
         {
             try
             {
+                string directory = Path.GetDirectoryName(this.userPresetFile);
+                if (!Directory.Exists(directory))
+                {
+                    Directory.CreateDirectory(directory);
+                }
+
                 using (FileStream strm = new FileStream(this.builtInPresetFile, FileMode.Create, FileAccess.Write))
                 {
                     Ser.Serialize(strm, this.presets.Where(p => p.IsBuildIn).ToList());
-                    strm.Close();
-                    strm.Dispose();
                 }
 
                 using (FileStream strm = new FileStream(this.userPresetFile, FileMode.Create, FileAccess.Write))
                 {
                     Ser.Serialize(strm, this.presets.Where(p => p.IsBuildIn == false).ToList());
-                    strm.Close();
-                    strm.Dispose();
                 }
             }
             catch (Exception exc)
