@@ -1,7 +1,11 @@
-﻿/*  QueryGeneratorUtility.cs $
-    This file is part of the HandBrake source code.
-    Homepage: <http://handbrake.fr/>.
-    It may be used under the terms of the GNU General Public License. */
+﻿// --------------------------------------------------------------------------------------------------------------------
+// <copyright file="QueryGeneratorUtility.cs" company="HandBrake Project (http://handbrake.fr)">
+//   This file is part of the HandBrake source code - It may be used under the terms of the GNU General Public License.
+// </copyright>
+// <summary>
+//   Generate a CLI Query for HandBrakeCLI
+// </summary>
+// --------------------------------------------------------------------------------------------------------------------
 
 namespace HandBrake.ApplicationServices.Utilities
 {
@@ -11,10 +15,8 @@ namespace HandBrake.ApplicationServices.Utilities
     using System.Globalization;
     using System.IO;
 
-    using HandBrake.ApplicationServices.Functions;
     using HandBrake.ApplicationServices.Model;
     using HandBrake.ApplicationServices.Model.Encoding;
-    using HandBrake.ApplicationServices.Services.Interfaces;
     using HandBrake.Interop.Model.Encoding;
     using HandBrake.Interop.Model.Encoding.x264;
 
@@ -23,26 +25,35 @@ namespace HandBrake.ApplicationServices.Utilities
     /// </summary>
     public class QueryGeneratorUtility
     {
-        /// <summary>
-        /// Backing field for the user settings service.
-        /// </summary>
-        private static readonly IUserSettingService UserSettingService = ServiceManager.UserSettingService;
-
         /// <summary>
         /// Generate a CLI Query for an EncodeTask Model object
         /// </summary>
         /// <param name="task">
         /// The task.
         /// </param>
+        /// <param name="previewScanCount">
+        /// The preview Scan Count.
+        /// </param>
+        /// <param name="verbosity">
+        /// The verbosity.
+        /// </param>
+        /// <param name="disableLibDvdNav">
+        /// The disable Lib Dvd Nav.
+        /// </param>
         /// <returns>
         /// A Cli Query
         /// </returns>
-        public static string GenerateQuery(EncodeTask task)
+        public static string GenerateQuery(EncodeTask task, int previewScanCount, int verbosity, bool disableLibDvdNav)
         {
+            if (string.IsNullOrEmpty(task.Source))
+            {
+                return "No source selected";
+            }
+
             string query = string.Empty;
-            query += SourceQuery(task, null, null);
+            query += SourceQuery(task, null, null, previewScanCount);
             query += DestinationQuery(task);
-            query += GenerateTabbedComponentsQuery(task, true);
+            query += GenerateTabbedComponentsQuery(task, true, verbosity, disableLibDvdNav);
 
             return query;
         }
@@ -59,15 +70,24 @@ namespace HandBrake.ApplicationServices.Utilities
         /// <param name="startAtPreview">
         /// The start At Preview.
         /// </param>
+        /// <param name="previewScanCount">
+        /// The preview Scan Count.
+        /// </param>
+        /// <param name="verbosity">
+        /// The verbosity.
+        /// </param>
+        /// <param name="disableLibDvdNav">
+        /// The disable Lib Dvd Nav.
+        /// </param>
         /// <returns>
         /// A Cli query suitable for generating a preview video.
         /// </returns>
-        public static string GeneratePreviewQuery(EncodeTask task, int duration, string startAtPreview)
+        public static string GeneratePreviewQuery(EncodeTask task, int duration, string startAtPreview, int previewScanCount, int verbosity, bool disableLibDvdNav)
         {
             string query = string.Empty;
-            query += SourceQuery(task, duration, startAtPreview);
+            query += SourceQuery(task, duration, startAtPreview, previewScanCount);
             query += DestinationQuery(task);
-            query += GenerateTabbedComponentsQuery(task, true);
+            query += GenerateTabbedComponentsQuery(task, true, verbosity, disableLibDvdNav);
 
             return query;
         }
@@ -83,10 +103,16 @@ namespace HandBrake.ApplicationServices.Utilities
         /// <param name="enableFilters">
         /// The enableFilters.
         /// </param>
+        /// <param name="verbosity">
+        /// The verbosity.
+        /// </param>
+        /// <param name="disableLibDvdNav">
+        /// The disable Lib Dvd Nav.
+        /// </param>
         /// <returns>
         /// The CLI query for the Tabbed section of the main window UI
         /// </returns>
-        private static string GenerateTabbedComponentsQuery(EncodeTask task, bool enableFilters)
+        private static string GenerateTabbedComponentsQuery(EncodeTask task, bool enableFilters, int verbosity, bool disableLibDvdNav)
         {
             string query = string.Empty;
 
@@ -116,7 +142,7 @@ namespace HandBrake.ApplicationServices.Utilities
             query += AdvancedQuery(task);
 
             // Extra Settings
-            query += ExtraSettings();
+            query += ExtraSettings(verbosity, disableLibDvdNav);
 
             return query;
         }
@@ -133,14 +159,20 @@ namespace HandBrake.ApplicationServices.Utilities
         /// <param name="preview">
         /// The preview.
         /// </param>
+        /// <param name="previewScanCount">
+        /// The preview Scan Count.
+        /// </param>
         /// <returns>
         /// A Cli Query as a string
         /// </returns>
-        private static string SourceQuery(EncodeTask task, int? duration, string preview)
+        private static string SourceQuery(EncodeTask task, int? duration, string preview, int previewScanCount)
         {
             string query = string.Empty;
 
-            query += string.Format(" -i \"{0}\"", task.Source);
+            query += task.Source.EndsWith("\\")
+                         ? string.Format(" -i \"{0}\\\\\"", task.Source.TrimEnd('\\'))
+                         : string.Format(" -i \"{0}\"", task.Source);
+
             query += string.Format(" -t {0}", task.Title);
             query += string.Format(" --angle {0}", task.Angle);
 
@@ -163,7 +195,7 @@ namespace HandBrake.ApplicationServices.Utilities
                     query += string.Format(" --start-at frame:{0} --stop-at frame:{1}", task.StartPoint, calculatedDuration);
                     break;
                 case PointToPointMode.Preview: // Preview
-                    query += " --previews " + UserSettingService.GetUserSetting<int>(ASUserSettingConstants.PreviewScanCount) + " ";
+                    query += " --previews " + previewScanCount + " ";
                     query += " --start-at-preview " + preview;
                     query += " --stop-at duration:" + duration + " ";
                     break;
@@ -183,14 +215,7 @@ namespace HandBrake.ApplicationServices.Utilities
         /// </returns>
         private static string DestinationQuery(EncodeTask task)
         {
-            string query = string.Empty;
-
-            if (task.PointToPointMode == PointToPointMode.Preview)
-                query += string.Format(" -o \"{0}\" ", task.Destination.Replace(".m", "_sample.m"));
-            else
-                query += string.Format(" -o \"{0}\" ", task.Destination);
-
-            return query;
+            return string.Format(" -o \"{0}\" ", task.Destination);
         }
 
         /// <summary>
@@ -236,11 +261,11 @@ namespace HandBrake.ApplicationServices.Utilities
 
             if (task.Anamorphic != Anamorphic.Strict)
             {
-                if (task.MaxWidth.HasValue) query += string.Format(" -X {0}", task.MaxWidth);
-                else if (task.Width.HasValue) query += string.Format(" -w {0}", task.Width);
+                //if (task.MaxWidth.HasValue) query += string.Format(" -X {0}", task.MaxWidth);
+                if (task.Width.HasValue && task.Width != 0) query += string.Format(" -w {0}", task.Width);
 
-                if (task.MaxWidth.HasValue) query += string.Format(" -Y {0}", task.MaxHeight);
-                else if (task.Height.HasValue) query += string.Format(" -h {0}", task.Height);
+                //if (task.MaxHeight.HasValue) query += string.Format(" -Y {0}", task.MaxHeight);
+                if (task.Height.HasValue && task.Height != 0) query += string.Format(" -l {0}", task.Height);
             }
 
             if (task.HasCropping)
@@ -318,6 +343,12 @@ namespace HandBrake.ApplicationServices.Utilities
                 case Decomb.Custom:
                     query += string.Format(" --decomb=\"{0}\"", task.CustomDecomb);
                     break;
+                case Decomb.Fast:
+                    query += " --decomb=\"fast\"";
+                    break;
+                case Decomb.Bob:
+                    query += " --decomb=\"bob\"";
+                    break;
                 default:
                     query += string.Empty;
                     break;
@@ -337,6 +368,10 @@ namespace HandBrake.ApplicationServices.Utilities
                 case Deinterlace.Custom:
                     query += string.Format(" --deinterlace=\"{0}\"", task.CustomDeinterlace);
                     break;
+
+                case Deinterlace.Bob:
+                    query += " --deinterlace=\"bob\"";
+                    break;
                 default:
                     query += string.Empty;
                     break;
@@ -361,7 +396,7 @@ namespace HandBrake.ApplicationServices.Utilities
                     break;
             }
 
-            if (task.Deblock != 4)
+            if (task.Deblock > 4)
                 query += string.Format(" --deblock={0}", task.Deblock);
 
             if (task.Grayscale)
@@ -386,7 +421,10 @@ namespace HandBrake.ApplicationServices.Utilities
             switch (task.VideoEncoder)
             {
                 case VideoEncoder.FFMpeg:
-                    query += " -e ffmpeg";
+                    query += " -e ffmpeg4";
+                    break;
+                case VideoEncoder.FFMpeg2:
+                    query += " -e ffmpeg2";
                     break;
                 case VideoEncoder.X264:
                     query += " -e x264";
@@ -399,8 +437,6 @@ namespace HandBrake.ApplicationServices.Utilities
                     break;
             }
 
-            double x264CqStep = UserSettingService.GetUserSetting<double>(ASUserSettingConstants.X264Step);
-
             switch (task.VideoEncodeRateType)
             {
                 case VideoEncodeRateType.AverageBitrate:
@@ -412,17 +448,14 @@ namespace HandBrake.ApplicationServices.Utilities
                     switch (task.VideoEncoder)
                     {
                         case VideoEncoder.FFMpeg:
-                            value = 31 - (task.Quality.Value - 1);
-                            query += string.Format(" -q {0}", value.ToString(CultureInfo.InvariantCulture));
+                        case VideoEncoder.FFMpeg2:
+                            query += string.Format(" -q {0}", task.Quality.Value.ToString(CultureInfo.InvariantCulture));
                             break;
                         case VideoEncoder.X264:
-                            value = 51 - (task.Quality.Value * x264CqStep);
-                            value = Math.Round(value, 2);
-                            query += string.Format(" -q {0}", value.ToString(CultureInfo.InvariantCulture));
+                            query += string.Format(" -q {0}", task.Quality.Value.ToString(CultureInfo.InvariantCulture));
                             break;
                         case VideoEncoder.Theora:
-                            value = task.Quality.Value;
-                            query += string.Format(" -q {0}", value.ToString(CultureInfo.InvariantCulture));
+                            query += string.Format(" -q {0}", task.Quality.Value.ToString(CultureInfo.InvariantCulture));
                             break;
                     }
                     break;
@@ -435,7 +468,7 @@ namespace HandBrake.ApplicationServices.Utilities
                 query += " -T ";
 
             if (task.Framerate.HasValue)
-                query += string.Format(" -r {0}", task.Framerate);
+                query += string.Format(" -r {0}", task.Framerate.Value.ToString(CultureInfo.InvariantCulture));
 
             switch (task.FramerateMode)
             {
@@ -477,6 +510,8 @@ namespace HandBrake.ApplicationServices.Utilities
             List<double> samplerates = new List<double>();
             List<int> bitrates = new List<int>();
             List<double> drcs = new List<double>();
+            List<double> gains = new List<double>();
+            List<string> trackNames = new List<string>();
 
             // No Audio
             if (audioTracks.Count == 0)
@@ -496,16 +531,22 @@ namespace HandBrake.ApplicationServices.Utilities
                 codecs.Add(track.Encoder);
 
                 // Audio Mixdown (-6)
-                mixdowns.Add(track.MixDown);
+                mixdowns.Add(track.IsPassthru ? Mixdown.None : track.MixDown);
 
                 // Sample Rate (-R)
-                samplerates.Add(track.SampleRate);
+                samplerates.Add(track.IsPassthru ? 0 : track.SampleRate);
 
                 // Audio Bitrate (-B)
-                bitrates.Add(track.Bitrate);
+                bitrates.Add(track.IsPassthru ? 0 : track.Bitrate);
 
                 // DRC (-D)
-                drcs.Add(track.DRC);
+                drcs.Add(track.IsPassthru ? 0 : track.DRC);
+
+                // Gain (--gain)
+                gains.Add(track.IsPassthru ? 0 : track.Gain);
+
+                // Audio Track Name (--aname)
+                trackNames.Add(track.TrackName);
             }
 
             // Audio Track (-a)
@@ -536,7 +577,7 @@ namespace HandBrake.ApplicationServices.Utilities
                     firstLoop = false;
                 }
                 else
-                    audioItems += "," + item;
+                    audioItems += "," +  Converters.GetCliAudioEncoder(item);
             }
             if (audioItems.Trim() != String.Empty)
                 query += " -E " + audioItems;
@@ -552,7 +593,7 @@ namespace HandBrake.ApplicationServices.Utilities
                     firstLoop = false;
                 }
                 else
-                    audioItems += "," + item;
+                    audioItems += "," + Converters.GetCliMixDown(item);
             }
             if (audioItems.Trim() != String.Empty)
                 query += " -6 " + audioItems;
@@ -562,13 +603,14 @@ namespace HandBrake.ApplicationServices.Utilities
             // Sample Rate (-R)
             foreach (double item in samplerates)
             {
+                string add = (item == 0.0) ? "Auto" : item.ToString();
                 if (firstLoop)
                 {
-                    audioItems = item.ToString();
+                    audioItems = add;
                     firstLoop = false;
                 }
                 else
-                    audioItems += "," + item;
+                    audioItems += "," + add;
             }
             if (audioItems.Trim() != String.Empty)
                 query += " -R " + audioItems;
@@ -606,39 +648,88 @@ namespace HandBrake.ApplicationServices.Utilities
             if (audioItems.Trim() != String.Empty)
                 query += " -D " + audioItems;
 
+            audioItems = string.Empty; // Reset for another pass.
+            firstLoop = true;
+
+            // Gain (--gain)
+            foreach (var itm in gains)
+            {
+                string item = itm.ToString(new CultureInfo("en-US"));
+                if (firstLoop)
+                {
+                    audioItems = item;
+                    firstLoop = false;
+                }
+                else
+                    audioItems += "," + item;
+            }
+            if (audioItems.Trim() != String.Empty)
+                query += " --gain " + audioItems;
+
+            audioItems = string.Empty; // Reset for another pass.
+            firstLoop = true;
+
+            // Audio Track Names (--aname)
+            bool foundTrackName = false;
+            foreach (string trackName in trackNames)
+            {
+                if (!string.IsNullOrEmpty(trackName))
+                {
+                    foundTrackName = true;
+                }
+
+                if (firstLoop)
+                {
+                    audioItems = string.IsNullOrEmpty(trackName) ? "\"\"" : string.Format("\"{0}\"", trackName.Trim());
+                    firstLoop = false;
+                }
+                else
+                    audioItems += "," + (string.IsNullOrEmpty(trackName) ? "\"\"" : string.Format("\"{0}\"", trackName.Trim()));
+            }
+            if (foundTrackName)
+                query += string.Format(" --aname={0}", audioItems);
+
             // Passthru Settings
             if (task.AllowedPassthruOptions != null)
             {
                 string fallbackEncoders = string.Empty;
 
-                if (task.AllowedPassthruOptions.AudioAllowAACPass)
+                if (task.AllowedPassthruOptions.AudioAllowAACPass != null && task.AllowedPassthruOptions.AudioAllowAACPass.Value)
                 {
                     fallbackEncoders += "aac";
                 }
 
-                if (task.AllowedPassthruOptions.AudioAllowAC3Pass)
+                if (task.AllowedPassthruOptions.AudioAllowAC3Pass != null && task.AllowedPassthruOptions.AudioAllowAC3Pass.Value)
                 {
                     fallbackEncoders += string.IsNullOrEmpty(fallbackEncoders) ? "ac3" : ",ac3";
                 }
 
-                if (task.AllowedPassthruOptions.AudioAllowDTSHDPass)
+                if (task.AllowedPassthruOptions.AudioAllowDTSHDPass != null && task.AllowedPassthruOptions.AudioAllowDTSHDPass.Value)
                 {
                     fallbackEncoders += string.IsNullOrEmpty(fallbackEncoders) ? "dtshd" : ",dtshd";
                 }
 
-                if (task.AllowedPassthruOptions.AudioAllowDTSPass)
+                if (task.AllowedPassthruOptions.AudioAllowDTSPass != null && task.AllowedPassthruOptions.AudioAllowDTSPass.Value)
                 {
                     fallbackEncoders += string.IsNullOrEmpty(fallbackEncoders) ? "dts" : ",dts";
                 }
 
-                if (task.AllowedPassthruOptions.AudioAllowMP3Pass)
+                if (task.AllowedPassthruOptions.AudioAllowMP3Pass != null && task.AllowedPassthruOptions.AudioAllowMP3Pass.Value)
                 {
                     fallbackEncoders += string.IsNullOrEmpty(fallbackEncoders) ? "mp3" : ",mp3";
                 }
 
                 if (!string.IsNullOrEmpty(fallbackEncoders))
                 {
-                    query += string.Format(" --audio-copy-mask {0}", fallbackEncoders);
+                    // Special Case, The CLI alredy defaults to ALL, so if all area selected, then don't need to set copy mask
+                    if (fallbackEncoders != "aac,ac3,dtshd,dts,mp3")
+                    {
+                        query += string.Format(" --audio-copy-mask {0}", fallbackEncoders);
+                    }
+                } 
+                else
+                {
+                    query += string.Format(" --audio-copy-mask none");  
                 }
 
                 query += string.Format(" --audio-fallback {0}", Converters.GetCliAudioEncoder(task.AllowedPassthruOptions.AudioEncoderFallback));
@@ -704,7 +795,7 @@ namespace HandBrake.ApplicationServices.Utilities
                         subCount++;
 
                         // Find --subtitle <string>
-                        if (item.Track.Contains("Foreign Audio Search"))
+                        if (item.SourceTrack.SubtitleType == SubtitleType.ForeignAudioSearch)
                             itemToAdd = "scan";
                         else
                         {
@@ -736,7 +827,7 @@ namespace HandBrake.ApplicationServices.Utilities
                     if (subtitleForced != string.Empty)
                         query += " --subtitle-forced=" + subtitleForced;
                     if (subtitleBurn != string.Empty)
-                        query += " --subtitle-burn=" + subtitleBurn;
+                        query += " --subtitle-burned=" + subtitleBurn;
                     if (subtitleDefault != string.Empty)
                         query += " --subtitle-default=" + subtitleDefault;
                 }
@@ -812,19 +903,37 @@ namespace HandBrake.ApplicationServices.Utilities
             {
                 string query = string.Empty; 
 
-                if (task.x264Preset != x264Preset.None)
+                if (task.X264Preset != x264Preset.Medium)
                 {
-                    query += string.Format("--x264-preset={0} ", task.x264Preset.ToString().ToLower().Replace(" ", string.Empty));
+                    query += string.Format(" --x264-preset={0} ", task.X264Preset.ToString().ToLower().Replace(" ", string.Empty));
                 }
 
-                if (task.x264Profile != x264Profile.None)
+                if (task.H264Profile != x264Profile.None)
                 {
-                    query += string.Format("--x264-profile={0} ", task.x264Profile.ToString().ToLower().Replace(" ", string.Empty));
+                    query += string.Format(" --x264-profile={0} ", task.H264Profile.ToString().ToLower().Replace(" ", string.Empty));
                 }
 
                 if (task.X264Tune != x264Tune.None)
                 {
-                    query += string.Format("--x264-tune={0} ", task.X264Tune.ToString().ToLower().Replace(" ", string.Empty));
+                    string tune = string.Empty;
+
+                    if (task.FastDecode)
+                    {
+                        tune = "fastdecode";
+                    }
+
+                    string tuneDropdown = task.X264Tune.ToString().ToLower().Replace(" ", string.Empty);
+                    if (task.X264Tune != x264Tune.None && !string.IsNullOrEmpty(tuneDropdown))
+                    {
+                        tune = string.IsNullOrEmpty(tune) ? tuneDropdown : string.Format(",{0}", tuneDropdown);  
+                    }
+
+                    query += string.Format(" --x264-tune=\"{0}\" ", tune);
+                }
+
+                if (task.H264Level != "Auto")
+                {
+                    query += string.Format(" --h264-level=\"{0}\" ", task.H264Level);
                 }
 
                 if (!string.IsNullOrEmpty(task.AdvancedEncoderOptions))
@@ -832,6 +941,11 @@ namespace HandBrake.ApplicationServices.Utilities
                     query += string.Format(" -x {0}", task.AdvancedEncoderOptions);
                 }
 
+                if (!string.IsNullOrEmpty(task.ExtraAdvancedArguments))
+                {
+                    query += string.Format(" -x {0}", task.ExtraAdvancedArguments);
+                }
+
                 return query;
             }
 
@@ -841,19 +955,24 @@ namespace HandBrake.ApplicationServices.Utilities
         /// <summary>
         /// Generate the Command Line Arguments for any additional advanced options.
         /// </summary>
+        /// <param name="verbosity">
+        /// The verbosity.
+        /// </param>
+        /// <param name="disableLibdvdNav">
+        /// The disable Libdvd Nav.
+        /// </param>
         /// <returns>
         /// A Cli Query as a string
         /// </returns>
-        private static string ExtraSettings()
+        private static string ExtraSettings(int verbosity, bool disableLibdvdNav)
         {
             string query = string.Empty;
 
             // Verbosity Level
-            int verbosity = UserSettingService.GetUserSetting<int>(ASUserSettingConstants.Verbosity);
-            query += string.Format(" --verbose= {0}", verbosity);
+            query += string.Format(" --verbose={0}", verbosity);
 
             // LibDVDNav
-            if (UserSettingService.GetUserSetting<bool>(ASUserSettingConstants.DisableLibDvdNav))
+            if (disableLibdvdNav)
                 query += " --no-dvdnav";
 
             return query;
@@ -872,7 +991,7 @@ namespace HandBrake.ApplicationServices.Utilities
         private static bool ChapterCsvSave(IEnumerable<ChapterMarker> chapters, string filePathName)
         {
             string csv = string.Empty;
-            int counter = 0;
+            int counter = 1;
 
             foreach (ChapterMarker name in chapters)
             {
