@@ -11,12 +11,15 @@ namespace HandBrake.Interop
 {
 	using System;
 	using System.Collections.Generic;
+	using System.Globalization;
 	using System.IO;
 	using System.Linq;
 	using System.Runtime.InteropServices;
 	using System.Windows.Media.Imaging;
 
+	using HandBrake.Interop.EventArgs;
 	using HandBrake.Interop.HbLib;
+	using HandBrake.Interop.Interfaces;
 	using HandBrake.Interop.Model;
 	using HandBrake.Interop.Model.Encoding;
 	using HandBrake.Interop.SourceData;
@@ -24,7 +27,7 @@ namespace HandBrake.Interop
 	/// <summary>
 	/// A wrapper for a HandBrake instance.
 	/// </summary>
-	public class HandBrakeInstance : IDisposable
+	public class HandBrakeInstance : IHandBrakeInstance, IDisposable
 	{
 		/// <summary>
 		/// The modulus for picture size when auto-sizing dimensions.
@@ -47,9 +50,14 @@ namespace HandBrake.Interop
 		private const string TurboX264Opts = "ref=1:subme=2:me=dia:analyse=none:trellis=0:no-fast-pskip=0:8x8dct=0:weightb=0";
 
 		/// <summary>
-		/// Default value to give as a minimum duration when scanning.
+		/// Lock for creation of handbrake instances;
 		/// </summary>
-		private const ulong DefaultMinDuration = 900000;
+		private static object instanceCreationLock = new object();
+
+		/// <summary>
+		/// True if a handbrake instance has been created.
+		/// </summary>
+		private static bool globalInitialized;
 
 		/// <summary>
 		/// The native handle to the HandBrake instance.
@@ -101,6 +109,11 @@ namespace HandBrake.Interop
 		/// </summary>
 		private List<IntPtr> encodeAllocatedMemory;
 
+		/// <summary>
+		/// A value indicating whether this object has been disposed or not.
+		/// </summary>
+		private bool disposed;
+
 		/// <summary>
 		/// Finalizes an instance of the HandBrakeInstance class.
 		/// </summary>
@@ -117,7 +130,7 @@ namespace HandBrake.Interop
 		/// <summary>
 		/// Fires when a scan has completed.
 		/// </summary>
-		public event EventHandler<EventArgs> ScanCompleted;
+		public event EventHandler<System.EventArgs> ScanCompleted;
 
 		/// <summary>
 		/// Fires for progress updates when encoding.
@@ -162,20 +175,20 @@ namespace HandBrake.Interop
 			}
 		}
 
-		/// <summary>
-		/// Call before app shutdown. Performs global cleanup.
-		/// </summary>
-		public static void DisposeGlobal()
-		{
-			HBFunctions.hb_global_close();
-		}
-
 		/// <summary>
 		/// Initializes this instance.
 		/// </summary>
 		/// <param name="verbosity">The code for the logging verbosity to use.</param>
 		public void Initialize(int verbosity)
 		{
+			lock (instanceCreationLock)
+			{
+				if (!globalInitialized)
+				{
+					globalInitialized = true;
+				}
+			}
+
 			HandBrakeUtils.RegisterLogger();
 			this.hbHandle = HBFunctions.hb_init(verbosity, update_check: 0);
 		}
@@ -185,9 +198,15 @@ namespace HandBrake.Interop
 		/// </summary>
 		/// <param name="path">The path to the video to scan.</param>
 		/// <param name="previewCount">The number of preview images to make.</param>
+		/// <param name="minDuration">The minimum duration of a title to show up on the scan.</param>
+		public void StartScan(string path, int previewCount, TimeSpan minDuration)
+		{
+			this.StartScan(path, previewCount, minDuration, 0);
+		}
+
 		public void StartScan(string path, int previewCount)
 		{
-			this.StartScan(path, previewCount, 0);
+			this.StartScan(path, previewCount, TimeSpan.FromSeconds(10), 0);
 		}
 
 		/// <summary>
@@ -198,17 +217,7 @@ namespace HandBrake.Interop
 		/// <param name="titleIndex">The title index to scan (1-based, 0 for all titles).</param>
 		public void StartScan(string path, int previewCount, int titleIndex)
 		{
-			this.previewCount = previewCount;
-			HBFunctions.hb_scan(this.hbHandle, path, titleIndex, previewCount, 1, DefaultMinDuration);
-			this.scanPollTimer = new System.Timers.Timer();
-			this.scanPollTimer.Interval = ScanPollIntervalMs;
-
-			// Lambda notation used to make sure we can view any JIT exceptions the method throws
-			this.scanPollTimer.Elapsed += (o, e) =>
-			{
-				this.PollScanProgress();
-			};
-			this.scanPollTimer.Start();
+			this.StartScan(path, previewCount, TimeSpan.Zero, titleIndex);
 		}
 
 		/// <summary>
@@ -230,31 +239,29 @@ namespace HandBrake.Interop
 		/// <returns>An image with the requested preview.</returns>
 		public BitmapImage GetPreview(EncodeJob job, int previewNumber)
 		{
-			hb_title_s title = this.GetOriginalTitle(job.Title);
+			IntPtr nativeJobPtr = HBFunctions.hb_job_init_by_index(this.hbHandle, this.GetTitleIndex(job.Title));
+			var nativeJob = InteropUtilities.ReadStructure<hb_job_s>(nativeJobPtr);
 
-			hb_job_s nativeJob = InteropUtilities.ReadStructure<hb_job_s>(title.job);
 			List<IntPtr> allocatedMemory = this.ApplyJob(ref nativeJob, job);
 			
 			// There are some problems with getting previews with deinterlacing. Disabling for now.
 			nativeJob.deinterlace = 0;
 
-			// Create a new job pointer from our modified job object
-			IntPtr newJob = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(hb_job_s)));
-			Marshal.StructureToPtr(nativeJob, newJob, false);
-			allocatedMemory.Add(newJob);
-
 			int outputWidth = nativeJob.width;
 			int outputHeight = nativeJob.height;
 			int imageBufferSize = outputWidth * outputHeight * 4;
 			IntPtr nativeBuffer = Marshal.AllocHGlobal(imageBufferSize);
 			allocatedMemory.Add(nativeBuffer);
-			HBFunctions.hb_set_job(this.hbHandle, job.Title, ref nativeJob);
-			HBFunctions.hb_get_preview_by_index(this.hbHandle, job.Title, previewNumber, nativeBuffer);
+			HBFunctions.hb_get_preview(this.hbHandle, ref nativeJob, previewNumber, nativeBuffer);
+
+			// We've used the job to get the preview. Clean up the job.
+			InteropUtilities.CloseJob(nativeJobPtr);
 
 			// Copy the filled image buffer to a managed array.
 			byte[] managedBuffer = new byte[imageBufferSize];
 			Marshal.Copy(nativeBuffer, managedBuffer, 0, imageBufferSize);
 
+			// We've copied the data out of unmanaged memory. Clean up that memory now.
 			InteropUtilities.FreeMemory(allocatedMemory);
 
 			var bitmap = new System.Drawing.Bitmap(outputWidth, outputHeight);
@@ -398,31 +405,14 @@ namespace HandBrake.Interop
 		/// for calculating bitrate when the target size would be wrong.</param>
 		public void StartEncode(EncodeJob job, bool preview, int previewNumber, int previewSeconds, double overallSelectedLengthSeconds)
 		{
+			EncodingProfile profile = job.EncodingProfile;
 			this.currentJob = job;
-			hb_job_s nativeJob = InteropUtilities.ReadStructure<hb_job_s>(this.GetOriginalTitle(job.Title).job);
+
+			IntPtr nativeJobPtr = HBFunctions.hb_job_init_by_index(this.hbHandle, this.GetTitleIndex(job.Title));
+			var nativeJob = InteropUtilities.ReadStructure<hb_job_s>(nativeJobPtr);
+
 			this.encodeAllocatedMemory = this.ApplyJob(ref nativeJob, job, preview, previewNumber, previewSeconds, overallSelectedLengthSeconds);
 
-			if (!preview && job.EncodingProfile.IncludeChapterMarkers)
-			{
-				Title title = this.GetTitle(job.Title);
-				int numChapters = title.Chapters.Count;
-
-				if (job.UseDefaultChapterNames)
-				{
-					for (int i = 0; i < numChapters; i++)
-					{
-						HBFunctions.hb_set_chapter_name(this.hbHandle, job.Title, i + 1, "Chapter " + (i + 1));
-					}
-				}
-				else
-				{
-					for (int i = 0; i < numChapters; i++)
-					{
-						HBFunctions.hb_set_chapter_name(this.hbHandle, job.Title, i + 1, job.CustomChapterNames[i]);
-					}
-				}
-			}
-
 			this.subtitleScan = false;
 			if (job.Subtitles.SourceSubtitles != null)
 			{
@@ -436,28 +426,34 @@ namespace HandBrake.Interop
 				}
 			}
 
-			string x264Options = job.EncodingProfile.X264Options ?? string.Empty;
+			string x264Options = profile.X264Options ?? string.Empty;
 			IntPtr originalX264Options = Marshal.StringToHGlobalAnsi(x264Options);
 			this.encodeAllocatedMemory.Add(originalX264Options);
 
-			if (!string.IsNullOrEmpty(job.EncodingProfile.X264Profile))
+			if (!string.IsNullOrEmpty(profile.X264Profile))
 			{
-				nativeJob.x264_profile = Marshal.StringToHGlobalAnsi(job.EncodingProfile.X264Profile);
-				this.encodeAllocatedMemory.Add(nativeJob.x264_profile);
+				nativeJob.h264_profile = Marshal.StringToHGlobalAnsi(profile.X264Profile);
+				this.encodeAllocatedMemory.Add(nativeJob.h264_profile);
 			}
 
-			if (!string.IsNullOrEmpty(job.EncodingProfile.X264Preset))
+			if (!string.IsNullOrEmpty(profile.X264Preset))
 			{
-				nativeJob.x264_preset = Marshal.StringToHGlobalAnsi(job.EncodingProfile.X264Preset);
+				nativeJob.x264_preset = Marshal.StringToHGlobalAnsi(profile.X264Preset);
 				this.encodeAllocatedMemory.Add(nativeJob.x264_preset);
 			}
 
-			if (!string.IsNullOrEmpty(job.EncodingProfile.X264Tune))
+			if (profile.X264Tunes != null && profile.X264Tunes.Count > 0)
 			{
-				nativeJob.x264_tune = Marshal.StringToHGlobalAnsi(job.EncodingProfile.X264Tune);
+				nativeJob.x264_tune = Marshal.StringToHGlobalAnsi(string.Join(",", profile.X264Tunes));
 				this.encodeAllocatedMemory.Add(nativeJob.x264_tune);
 			}
 
+			if (!string.IsNullOrEmpty(job.EncodingProfile.H264Level))
+			{
+				nativeJob.h264_level = Marshal.StringToHGlobalAnsi(job.EncodingProfile.H264Level);
+				this.encodeAllocatedMemory.Add(nativeJob.h264_level);
+			}
+
 			if (this.subtitleScan)
 			{
 				// If we need to scan subtitles, enqueue a pre-processing job to do that.
@@ -510,6 +506,9 @@ namespace HandBrake.Interop
 
 			HBFunctions.hb_start(this.hbHandle);
 
+			// Should be safe to clean up the job we started with; a copy is in the queue now.
+			InteropUtilities.CloseJob(nativeJobPtr);
+
 			this.encodePollTimer = new System.Timers.Timer();
 			this.encodePollTimer.Interval = EncodePollIntervalMs;
 
@@ -568,9 +567,10 @@ namespace HandBrake.Interop
 		/// <param name="parHeight">The pixel aspect Y number.</param>
 		public void GetSize(EncodeJob job, out int width, out int height, out int parWidth, out int parHeight)
 		{
+			Title title = this.GetTitle(job.Title);
+
 			if (job.EncodingProfile.Anamorphic == Anamorphic.None)
 			{
-				Title title = this.GetTitle(job.Title);
 				Size storageDimensions = CalculateNonAnamorphicOutput(job.EncodingProfile, title);
 
 				width = storageDimensions.Width;
@@ -582,28 +582,34 @@ namespace HandBrake.Interop
 				return;
 			}
 
-			var nativeJob = InteropUtilities.ReadStructure<hb_job_s>(this.GetOriginalTitle(job.Title).job);
-			List<IntPtr> allocatedMemory = this.ApplyJob(ref nativeJob, job);
+			IntPtr nativeJobPtr = HBFunctions.hb_job_init_by_index(this.hbHandle, this.GetTitleIndex(title));
+			var nativeJob = InteropUtilities.ReadStructure<hb_job_s>(nativeJobPtr);
 
-			int refWidth = 0;
-			int refHeight = 0;
-			int refParWidth = 0;
-			int refParHeight = 0;
-			HBFunctions.hb_set_job(this.hbHandle, job.Title, ref nativeJob);
-			HBFunctions.hb_set_anamorphic_size_by_index(this.hbHandle, job.Title, ref refWidth, ref refHeight, ref refParWidth, ref refParHeight);
+			List<IntPtr> allocatedMemory = this.ApplyJob(ref nativeJob, job);
 			InteropUtilities.FreeMemory(allocatedMemory);
 
-			width = refWidth;
-			height = refHeight;
-			parWidth = refParWidth;
-			parHeight = refParHeight;
+			InteropUtilities.CloseJob(nativeJobPtr);
+
+			// During the ApplyJob call, it modified nativeJob to have the correct width, height and PAR.
+			// We use those for the size.
+			width = nativeJob.width;
+			height = nativeJob.height;
+			parWidth = nativeJob.anamorphic.par_width;
+			parHeight = nativeJob.anamorphic.par_height;
 		}
 
+
+
 		/// <summary>
 		/// Frees any resources associated with this object.
 		/// </summary>
 		public void Dispose()
 		{
+			if (this.disposed)
+			{
+				return;
+			}
+
 			this.Dispose(true);
 			GC.SuppressFinalize(this);
 		}
@@ -624,6 +630,8 @@ namespace HandBrake.Interop
 			Marshal.WriteIntPtr(handlePtr, this.hbHandle);
 			HBFunctions.hb_close(handlePtr);
 			Marshal.FreeHGlobal(handlePtr);
+
+			this.disposed = true;
 		}
 
 		/// <summary>
@@ -641,13 +649,17 @@ namespace HandBrake.Interop
 			int height = profile.Height;
 
 			Cropping crop;
-			if (profile.CustomCropping)
+			switch (profile.CroppingType)
 			{
-				crop = profile.Cropping;
-			}
-			else
-			{
-				crop = title.AutoCropDimensions;
+				case CroppingType.Automatic:
+					crop = title.AutoCropDimensions;
+					break;
+				case CroppingType.Custom:
+					crop = profile.Cropping;
+					break;
+				default:
+					crop = new Cropping();
+					break;
 			}
 
 			sourceWidth -= crop.Left;
@@ -720,6 +732,28 @@ namespace HandBrake.Interop
 			return modulus * ((number + modulus / 2) / modulus);
 		}
 
+		/// <summary>
+		/// Starts a scan of the given path.
+		/// </summary>
+		/// <param name="path">The path of the video to scan.</param>
+		/// <param name="previewCount">The number of previews to make on each title.</param>
+		/// <param name="minDuration">The minimum duration of a title to show up on the scan.</param>
+		/// <param name="titleIndex">The title index to scan (1-based, 0 for all titles).</param>
+		private void StartScan(string path, int previewCount, TimeSpan minDuration, int titleIndex)
+		{
+			this.previewCount = previewCount;
+			HBFunctions.hb_scan(this.hbHandle, path, titleIndex, previewCount, 1, (ulong)(minDuration.TotalSeconds * 90000));
+			this.scanPollTimer = new System.Timers.Timer();
+			this.scanPollTimer.Interval = ScanPollIntervalMs;
+
+			// Lambda notation used to make sure we can view any JIT exceptions the method throws
+			this.scanPollTimer.Elapsed += (o, e) =>
+			{
+				this.PollScanProgress();
+			};
+			this.scanPollTimer.Start();
+		}
+
 		/// <summary>
 		/// Checks the status of the ongoing scan.
 		/// </summary>
@@ -732,17 +766,25 @@ namespace HandBrake.Interop
 			{
 				if (this.ScanProgress != null)
 				{
-					int currentTitle = state.param.scanning.title_cur;
-					int totalTitles = state.param.scanning.title_count;
-					this.ScanProgress(this, new ScanProgressEventArgs { CurrentTitle = currentTitle, Titles = totalTitles });
+					hb_state_scanning_anon scanningState = state.param.scanning;
+
+					this.ScanProgress(this, new ScanProgressEventArgs
+						{
+							Progress = scanningState.progress,
+							CurrentPreview = scanningState.preview_cur,
+							Previews = scanningState.preview_count,
+							CurrentTitle = scanningState.title_cur,
+							Titles = scanningState.title_count
+						});
 				}
 			}
 			else if (state.state == NativeConstants.HB_STATE_SCANDONE)
 			{
 				this.titles = new List<Title>();
 
-				IntPtr listPtr = HBFunctions.hb_get_titles(this.hbHandle);
-				this.originalTitles = InteropUtilities.ConvertList<hb_title_s>(listPtr);
+				IntPtr titleSetPtr = HBFunctions.hb_get_title_set(this.hbHandle);
+				hb_title_set_s titleSet = InteropUtilities.ReadStructure<hb_title_set_s>(titleSetPtr);
+				this.originalTitles = titleSet.list_title.ToList<hb_title_s>();
 
 				foreach (hb_title_s title in this.originalTitles)
 				{
@@ -752,8 +794,7 @@ namespace HandBrake.Interop
 
 				if (this.originalTitles.Count > 0)
 				{
-					var nativeJob = InteropUtilities.ReadStructure<hb_job_s>(this.originalTitles[0].job);
-					this.featureTitle = nativeJob.feature;
+					this.featureTitle = titleSet.feature;
 				}
 				else
 				{
@@ -764,7 +805,7 @@ namespace HandBrake.Interop
 
 				if (this.ScanCompleted != null)
 				{
-					this.ScanCompleted(this, new EventArgs());
+					this.ScanCompleted(this, new System.EventArgs());
 				}
 			}
 		}
@@ -908,8 +949,9 @@ namespace HandBrake.Interop
 			{
 				switch (job.RangeType)
 				{
+					case VideoRangeType.All:
+						break;
 					case VideoRangeType.Chapters:
-
 						if (job.ChapterStart > 0 && job.ChapterEnd > 0)
 						{
 							nativeJob.chapter_start = job.ChapterStart;
@@ -928,9 +970,13 @@ namespace HandBrake.Interop
 							throw new ArgumentException("Seconds range " + job.SecondsStart + "-" + job.SecondsEnd + " is invalid.", "job");
 						}
 
-						// For some reason "pts_to_stop" actually means the number of pts to stop AFTER the start point.
-						nativeJob.pts_to_start = (int)(job.SecondsStart * 90000);
-						nativeJob.pts_to_stop = (int)((job.SecondsEnd - job.SecondsStart) * 90000);
+						// If they've selected the "full" title duration, leave off the arguments to make it clean
+						if (job.SecondsStart > 0 || job.SecondsEnd < title.Duration.TotalSeconds)
+						{
+							// For some reason "pts_to_stop" actually means the number of pts to stop AFTER the start point.
+							nativeJob.pts_to_start = (int) (job.SecondsStart * 90000);
+							nativeJob.pts_to_stop = (int) ((job.SecondsEnd - job.SecondsStart) * 90000);
+						}
 						break;
 					case VideoRangeType.Frames:
 						if (job.FramesStart < 0 || job.FramesEnd < 0 || job.FramesStart >= job.FramesEnd)
@@ -945,55 +991,50 @@ namespace HandBrake.Interop
 				}
 			}
 
+			// Chapter markers
 			nativeJob.chapter_markers = profile.IncludeChapterMarkers ? 1 : 0;
 
-			Cropping crop;
+			List<IntPtr> nativeChapters = nativeJob.list_chapter.ToIntPtrList();
 
-			if (profile.CustomCropping)
+			if (!preview && profile.IncludeChapterMarkers)
 			{
-				crop = profile.Cropping;
-			}
-			else
-			{
-				crop = title.AutoCropDimensions;
+				int numChapters = title.Chapters.Count;
+
+				if (job.UseDefaultChapterNames)
+				{
+					for (int i = 0; i < numChapters; i++)
+					{
+						if (i < nativeChapters.Count)
+						{
+							HBFunctions.hb_chapter_set_title(nativeChapters[i], "Chapter " + (i + 1));
+						}
+					}
+				}
+				else
+				{
+					for (int i = 0; i < numChapters; i++)
+					{
+						if (i < nativeChapters.Count && i < job.CustomChapterNames.Count)
+						{
+							HBFunctions.hb_chapter_set_title(nativeChapters[i], job.CustomChapterNames[i]);
+						}
+					}
+				}
 			}
 
+			Cropping crop = GetCropping(profile, title);
+
 			nativeJob.crop[0] = crop.Top;
 			nativeJob.crop[1] = crop.Bottom;
 			nativeJob.crop[2] = crop.Left;
 			nativeJob.crop[3] = crop.Right;
 
-			var filterList = new List<IntPtr>();
-			if (profile.Deinterlace != Deinterlace.Off)
-			{
-				nativeJob.deinterlace = 1;
-				string settings = null;
+			var filterList = new List<hb_filter_object_s>();
 
-				switch (profile.Deinterlace)
-				{
-					case Deinterlace.Fast:
-						settings = "-1";
-						break;
-					case Deinterlace.Slow:
-						settings = "2";
-						break;
-					case Deinterlace.Slower:
-						settings = "0";
-						break;
-					case Deinterlace.Custom:
-						settings = profile.CustomDeinterlace;
-						break;
-					default:
-						break;
-				}
-
-				this.AddFilter(filterList, NativeConstants.HB_FILTER_DEINTERLACE, settings, allocatedMemory);
-			}
-			else
-			{
-				nativeJob.deinterlace = 0;
-			}
+			// FILTERS: These must be added in the correct order since we cannot depend on the automatic ordering in hb_add_filter . Ordering is determined
+			// by the order they show up in the filters enum.
 
+			// Detelecine
 			if (profile.Detelecine != Detelecine.Off)
 			{
 				string settings = null;
@@ -1002,25 +1043,111 @@ namespace HandBrake.Interop
 					settings = profile.CustomDetelecine;
 				}
 
-				this.AddFilter(filterList, NativeConstants.HB_FILTER_DETELECINE, settings, allocatedMemory);
+				this.AddFilter(filterList, (int)hb_filter_ids.HB_FILTER_DETELECINE, settings, allocatedMemory);
 			}
 
+			// Decomb
 			if (profile.Decomb != Decomb.Off)
 			{
 				string settings = null;
-				if (profile.Decomb == Decomb.Custom)
+				switch (profile.Decomb)
 				{
-					settings = profile.CustomDecomb;
+					case Decomb.Default:
+						break;
+					case Decomb.Fast:
+						settings = "7:2:6:9:1:80";
+						break;
+					case Decomb.Bob:
+						settings = "455";
+						break;
+					case Decomb.Custom:
+						settings = profile.CustomDecomb;
+						break;
+					default:
+						break;
 				}
 
-				this.AddFilter(filterList, NativeConstants.HB_FILTER_DECOMB, settings, allocatedMemory);
+				this.AddFilter(filterList, (int)hb_filter_ids.HB_FILTER_DECOMB, settings, allocatedMemory);
 			}
 
+			// Deinterlace
+			if (profile.Deinterlace != Deinterlace.Off)
+			{
+				nativeJob.deinterlace = 1;
+				string settings = null;
+
+				switch (profile.Deinterlace)
+				{
+					case Deinterlace.Fast:
+						settings = "0";
+						break;
+					case Deinterlace.Slow:
+						settings = "1";
+						break;
+					case Deinterlace.Slower:
+						settings = "3";
+						break;
+					case Deinterlace.Bob:
+						settings = "15";
+						break;
+					case Deinterlace.Custom:
+						settings = profile.CustomDeinterlace;
+						break;
+					default:
+						break;
+				}
+
+				this.AddFilter(filterList, (int)hb_filter_ids.HB_FILTER_DEINTERLACE, settings, allocatedMemory);
+			}
+			else
+			{
+				nativeJob.deinterlace = 0;
+			}
+
+			// VFR
+			if (profile.Framerate == 0)
+			{
+				if (profile.ConstantFramerate)
+				{
+					// CFR with "Same as Source". Use the title rate
+					nativeJob.cfr = 1;
+					nativeJob.vrate = originalTitle.rate;
+					nativeJob.vrate_base = originalTitle.rate_base;
+				}
+				else
+				{
+					// Pure VFR "Same as Source"
+					nativeJob.cfr = 0;
+				}
+			}
+			else
+			{
+				// Specified framerate
+				if (profile.ConstantFramerate)
+				{
+					// Mark as pure CFR
+					nativeJob.cfr = 1;
+				}
+				else
+				{
+					// Mark as peak framerate
+					nativeJob.cfr = 2;
+				}
+
+				nativeJob.vrate = 27000000;
+				nativeJob.vrate_base = Converters.FramerateToVrate(profile.Framerate);
+			}
+
+			string vfrSettings = string.Format(CultureInfo.InvariantCulture, "{0}:{1}:{2}", nativeJob.cfr, nativeJob.vrate, nativeJob.vrate_base);
+			this.AddFilter(filterList, (int)hb_filter_ids.HB_FILTER_VFR, vfrSettings, allocatedMemory);
+
+			// Deblock
 			if (profile.Deblock > 0)
 			{
-				this.AddFilter(filterList, NativeConstants.HB_FILTER_DEBLOCK, profile.Deblock.ToString(), allocatedMemory);
+				this.AddFilter(filterList, (int)hb_filter_ids.HB_FILTER_DEBLOCK, profile.Deblock.ToString(CultureInfo.InvariantCulture), allocatedMemory);
 			}
 
+			// Denoise
 			if (profile.Denoise != Denoise.Off)
 			{
 				string settings = null;
@@ -1042,13 +1169,10 @@ namespace HandBrake.Interop
 						break;
 				}
 
-				this.AddFilter(filterList, NativeConstants.HB_FILTER_DENOISE, settings, allocatedMemory);
+				this.AddFilter(filterList, (int)hb_filter_ids.HB_FILTER_DENOISE, settings, allocatedMemory);
 			}
 
-			NativeList filterListNative = InteropUtilities.CreateIntPtrList(filterList);
-			nativeJob.filters = filterListNative.ListPtr;
-			allocatedMemory.AddRange(filterListNative.AllocatedMemory);
-
+			// Crop/scale
 			int width = profile.Width;
 			int height = profile.Height;
 
@@ -1075,6 +1199,11 @@ namespace HandBrake.Interop
 				height = profile.MaxHeight;
 			}
 
+			// The job width can sometimes start not clean, due to interference from
+			// preview generation. We reset it here to allow good calculations.
+			nativeJob.width = width;
+			nativeJob.height = height;
+
 			nativeJob.grayscale = profile.Grayscale ? 1 : 0;
 
 			switch (profile.Anamorphic)
@@ -1087,12 +1216,31 @@ namespace HandBrake.Interop
 					height = outputSize.Height;
 
 					nativeJob.anamorphic.keep_display_aspect = profile.KeepDisplayAspect ? 1 : 0;
+
+					nativeJob.width = width;
+					nativeJob.height = height;
+
+					nativeJob.maxWidth = profile.MaxWidth;
+					nativeJob.maxHeight = profile.MaxHeight;
+
 					break;
 				case Anamorphic.Strict:
 					nativeJob.anamorphic.mode = 1;
+
+					nativeJob.anamorphic.par_width = title.ParVal.Width;
+					nativeJob.anamorphic.par_height = title.ParVal.Height;
 					break;
 				case Anamorphic.Loose:
 					nativeJob.anamorphic.mode = 2;
+
+					nativeJob.modulus = profile.Modulus;
+
+					nativeJob.width = width;
+
+					nativeJob.maxWidth = profile.MaxWidth;
+
+					nativeJob.anamorphic.par_width = title.ParVal.Width;
+					nativeJob.anamorphic.par_height = title.ParVal.Height;
 					break;
 				case Anamorphic.Custom:
 					nativeJob.anamorphic.mode = 3;
@@ -1126,10 +1274,12 @@ namespace HandBrake.Interop
 							nativeJob.anamorphic.dar_height = height;
 							nativeJob.anamorphic.keep_display_aspect = 1;
 						}
-
-						nativeJob.anamorphic.dar_width = profile.DisplayWidth;
-						nativeJob.anamorphic.dar_height = height;
-						nativeJob.anamorphic.keep_display_aspect = profile.KeepDisplayAspect ? 1 : 0;
+						else
+						{
+							nativeJob.anamorphic.dar_width = profile.DisplayWidth;
+							nativeJob.anamorphic.dar_height = height;
+							nativeJob.anamorphic.keep_display_aspect = profile.KeepDisplayAspect ? 1 : 0;
+						}
 					}
 					else
 					{
@@ -1138,16 +1288,43 @@ namespace HandBrake.Interop
 						nativeJob.anamorphic.keep_display_aspect = 0;
 					}
 
+					nativeJob.width = width;
+					nativeJob.height = height;
+
+					nativeJob.maxWidth = profile.MaxWidth;
+					nativeJob.maxHeight = profile.MaxHeight;
+
 					break;
 				default:
 					break;
 			}
 
-			nativeJob.width = width;
-			nativeJob.height = height;
+			// Need to fix up values before adding crop/scale filter
+			if (profile.Anamorphic != Anamorphic.None)
+			{
+				int anamorphicWidth = 0, anamorphicHeight = 0, anamorphicParWidth = 0, anamorphicParHeight = 0;
+
+				HBFunctions.hb_set_anamorphic_size(ref nativeJob, ref anamorphicWidth, ref anamorphicHeight, ref anamorphicParWidth, ref anamorphicParHeight);
+				nativeJob.width = anamorphicWidth;
+				nativeJob.height = anamorphicHeight;
+				nativeJob.anamorphic.par_width = anamorphicParWidth;
+				nativeJob.anamorphic.par_height = anamorphicParHeight;
+			}
+
+			string cropScaleSettings = string.Format(
+				CultureInfo.InvariantCulture,
+				"{0}:{1}:{2}:{3}:{4}:{5}",
+				nativeJob.width,
+				nativeJob.height,
+				nativeJob.crop[0],
+				nativeJob.crop[1],
+				nativeJob.crop[2],
+				nativeJob.crop[3]);
+			this.AddFilter(filterList, (int)hb_filter_ids.HB_FILTER_CROP_SCALE, cropScaleSettings, allocatedMemory);
+
+
+
 
-			nativeJob.maxWidth = profile.MaxWidth;
-			nativeJob.maxHeight = profile.MaxHeight;
 
 			HBVideoEncoder videoEncoder = Encoders.VideoEncoders.FirstOrDefault(e => e.ShortName == profile.VideoEncoder);
 			if (videoEncoder == null)
@@ -1157,37 +1334,25 @@ namespace HandBrake.Interop
 
 			nativeJob.vcodec = videoEncoder.Id;
 
-			if (profile.Framerate == 0)
-			{
-				nativeJob.cfr = 0;
-			}
-			else
-			{
-				if (profile.PeakFramerate)
-				{
-					nativeJob.cfr = 2;
-				}
-				else
-				{
-					nativeJob.cfr = 1;
-				}
 
-				nativeJob.vrate = 27000000;
-				nativeJob.vrate_base = Converters.FramerateToVrate(profile.Framerate);
-			}
 
-			// vfr
 			// areBframes
 			// color_matrix
-			List<hb_audio_s> titleAudio = InteropUtilities.ConvertList<hb_audio_s>(originalTitle.list_audio);
+			List<hb_audio_s> titleAudio = originalTitle.list_audio.ToList<hb_audio_s>();
 			
 			var audioList = new List<hb_audio_s>();
 			int numTracks = 0;
 
 			List<Tuple<AudioEncoding, int>> outputTrackList = this.GetOutputTracks(job, title);
 
-			if (profile.AudioEncoderFallback != null)
+			if (!string.IsNullOrEmpty(profile.AudioEncoderFallback))
 			{
+				HBAudioEncoder audioEncoder = Encoders.GetAudioEncoder(profile.AudioEncoderFallback);
+				if (audioEncoder == null)
+				{
+					throw new ArgumentException("Unrecognized fallback audio encoder: " + profile.AudioEncoderFallback);
+				}
+
 				nativeJob.acodec_fallback = Encoders.GetAudioEncoder(profile.AudioEncoderFallback).Id;
 			}
 
@@ -1225,7 +1390,7 @@ namespace HandBrake.Interop
 			{
 				if (job.Subtitles.SourceSubtitles != null && job.Subtitles.SourceSubtitles.Count > 0)
 				{
-					List<hb_subtitle_s> titleSubtitles = InteropUtilities.ConvertList<hb_subtitle_s>(originalTitle.list_subtitle);
+					List<hb_subtitle_s> titleSubtitles = originalTitle.list_subtitle.ToList<hb_subtitle_s>();
 
 					foreach (SourceSubtitle sourceSubtitle in job.Subtitles.SourceSubtitles)
 					{
@@ -1251,7 +1416,7 @@ namespace HandBrake.Interop
 							subtitleConfig.force = sourceSubtitle.Forced ? 1 : 0;
 							subtitleConfig.default_track = sourceSubtitle.Default ? 1 : 0;
 
-							bool supportsBurn = nativeSubtitle.source == hb_subtitle_s_subsource.VOBSUB || nativeSubtitle.source == hb_subtitle_s_subsource.SSASUB;
+							bool supportsBurn = nativeSubtitle.source == hb_subtitle_s_subsource.VOBSUB || nativeSubtitle.source == hb_subtitle_s_subsource.SSASUB || nativeSubtitle.source == hb_subtitle_s_subsource.PGSSUB;
 							if (supportsBurn && sourceSubtitle.BurnedIn)
 							{
 								subtitleConfig.dest = hb_subtitle_config_s_subdest.RENDERSUB;
@@ -1288,8 +1453,17 @@ namespace HandBrake.Interop
 						}
 					}
 				}
+
+				bool hasBurnedSubtitle = job.Subtitles.SourceSubtitles.Any(s => s.BurnedIn);
+				if (hasBurnedSubtitle)
+				{
+					this.AddFilter(filterList, (int)hb_filter_ids.HB_FILTER_RENDER_SUB, string.Format(CultureInfo.InvariantCulture, "{0}:{1}:{2}:{3}", crop.Top, crop.Bottom, crop.Left, crop.Right), allocatedMemory);
+				}
 			}
 
+			// Construct final filter list
+			nativeJob.list_filter = this.ConvertFilterListToNative(filterList, allocatedMemory).ListPtr;
+
 			if (profile.OutputFormat == Container.Mp4)
 			{
 				nativeJob.mux = NativeConstants.HB_MUX_MP4;
@@ -1378,26 +1552,77 @@ namespace HandBrake.Interop
 		/// <summary>
 		/// Adds a filter to the given filter list.
 		/// </summary>
-		/// <param name="filterList">The filter list to add to.</param>
+		/// <param name="filterList">The list to add the filter to.</param>
 		/// <param name="filterType">The type of filter.</param>
 		/// <param name="settings">Settings for the filter.</param>
 		/// <param name="allocatedMemory">The list of allocated memory.</param>
-		private void AddFilter(List<IntPtr> filterList, int filterType, string settings, List<IntPtr> allocatedMemory)
+		private void AddFilter(List<hb_filter_object_s> filterList, int filterType, string settings, List<IntPtr> allocatedMemory)
 		{
 			IntPtr settingsNativeString = Marshal.StringToHGlobalAnsi(settings);
-			filterList.Add(HBFunctions.hb_get_filter_object(filterType, settingsNativeString));
+			hb_filter_object_s filter = InteropUtilities.ReadStructure<hb_filter_object_s>(HBFunctions.hb_filter_init(filterType));
+			filter.settings = settingsNativeString;
 
 			allocatedMemory.Add(settingsNativeString);
+			filterList.Add(filter);
 		}
 
 		/// <summary>
-		/// Gets the title, given the 1-based index.
+		/// Converts the given filter list to a native list.
 		/// </summary>
-		/// <param name="titleIndex">The index of the title (1-based).</param>
-		/// <returns>The requested Title.</returns>
-		private Title GetTitle(int titleIndex)
+		/// <remarks>Sorts the list by filter ID before converting to a native list, as HB expects it that way.
+		/// The list memory itself will be added to the allocatedMemory list.</remarks>
+		/// <param name="filterList">The filter list to convert.</param>
+		/// <param name="allocatedMemory">The list of allocated memory to add to.</param>
+		/// <returns>The converted list.</returns>
+		private NativeList ConvertFilterListToNative(List<hb_filter_object_s> filterList, List<IntPtr> allocatedMemory)
 		{
-			return this.Titles.SingleOrDefault(title => title.TitleNumber == titleIndex);
+			var filterPtrList = new List<IntPtr>();
+
+			var sortedList = filterList.OrderBy(f => f.id);
+			foreach (var filter in sortedList)
+			{
+				IntPtr filterPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(hb_filter_object_s)));
+				Marshal.StructureToPtr(filter, filterPtr, false);
+
+				allocatedMemory.Add(filterPtr);
+				filterPtrList.Add(filterPtr);
+			}
+
+			NativeList filterListNative = InteropUtilities.CreateIntPtrList(filterPtrList);
+			allocatedMemory.AddRange(filterListNative.AllocatedMemory);
+
+			return filterListNative;
+		}
+
+		/// <summary>
+		/// Gets the title, given the 1-based title number.
+		/// </summary>
+		/// <param name="titleNumber">The number of the title (1-based).</param>
+		/// <returns>The requested Title.</returns>
+		private Title GetTitle(int titleNumber)
+		{
+			return this.Titles.SingleOrDefault(title => title.TitleNumber == titleNumber);
+		}
+
+		/// <summary>
+		/// Gets the 1-based title index of the given title.
+		/// </summary>
+		/// <param name="titleNumber">The 1-based title title number.</param>
+		/// <returns>The 1-based title index.</returns>
+		private int GetTitleIndex(int titleNumber)
+		{
+			Title title = this.GetTitle(titleNumber);
+			return this.GetTitleIndex(title);
+		}
+
+		/// <summary>
+		/// Gets the 1-based title index of the given title.
+		/// </summary>
+		/// <param name="title">The title to look up</param>
+		/// <returns>The 1-based title index of the given title.</returns>
+		private int GetTitleIndex(Title title)
+		{
+			return this.Titles.IndexOf(title) + 1;
 		}
 
 		/// <summary>
@@ -1508,9 +1733,11 @@ namespace HandBrake.Interop
 			var newTitle = new Title
 			{
 				TitleNumber = title.index,
+				Playlist = title.playlist,
 				Resolution = new Size(title.width, title.height),
 				ParVal = new Size(title.pixel_aspect_width, title.pixel_aspect_height),
-				Duration = TimeSpan.FromSeconds(title.duration / 90000),
+				Duration = Converters.PtsToTimeSpan(title.duration),
+				DurationPts = title.duration,
 				AutoCropDimensions = new Cropping
 				{
 					Top = title.crop[0],
@@ -1521,7 +1748,9 @@ namespace HandBrake.Interop
 				AspectRatio = title.aspect,
 				AngleCount = title.angle_count,
 				VideoCodecName = title.video_codec_name,
-				Framerate = ((double)title.rate) / title.rate_base
+				Framerate = ((double)title.rate) / title.rate_base,
+				FramerateNumerator = title.rate,
+				FramerateDenominator = title.rate_base
 			};
 
 			switch (title.type)
@@ -1538,7 +1767,7 @@ namespace HandBrake.Interop
 			}
 
 			int currentSubtitleTrack = 1;
-			List<hb_subtitle_s> subtitleList = InteropUtilities.ConvertList<hb_subtitle_s>(title.list_subtitle);
+			List<hb_subtitle_s> subtitleList = title.list_subtitle.ToList<hb_subtitle_s>();
 			foreach (hb_subtitle_s subtitle in subtitleList)
 			{
 				var newSubtitle = new Subtitle
@@ -1580,6 +1809,9 @@ namespace HandBrake.Interop
 					case hb_subtitle_s_subsource.VOBSUB:
 						newSubtitle.SubtitleSource = SubtitleSource.VobSub;
 						break;
+					case hb_subtitle_s_subsource.PGSSUB:
+						newSubtitle.SubtitleSource = SubtitleSource.PGS;
+						break;
 					default:
 						break;
 				}
@@ -1590,7 +1822,7 @@ namespace HandBrake.Interop
 			}
 
 			int currentAudioTrack = 1;
-			List<hb_audio_s> audioList = InteropUtilities.ConvertList<hb_audio_s>(title.list_audio);
+			List<hb_audio_s> audioList = title.list_audio.ToList<hb_audio_s>();
 			foreach (hb_audio_s audio in audioList)
 			{
 				var newAudio = new AudioTrack
@@ -1611,13 +1843,14 @@ namespace HandBrake.Interop
 				currentAudioTrack++;
 			}
 
-			List<hb_chapter_s> chapterList = InteropUtilities.ConvertList<hb_chapter_s>(title.list_chapter);
+			List<hb_chapter_s> chapterList = title.list_chapter.ToList<hb_chapter_s>();
 			foreach (hb_chapter_s chapter in chapterList)
 			{
 				var newChapter = new Chapter
 				{
 					ChapterNumber = chapter.index,
-					Duration = TimeSpan.FromSeconds(((double)chapter.duration) / 90000)
+					Duration = Converters.PtsToTimeSpan(chapter.duration),
+					DurationPts = chapter.duration
 				};
 
 				newTitle.Chapters.Add(newChapter);
@@ -1625,5 +1858,29 @@ namespace HandBrake.Interop
 
 			return newTitle;
 		}
+
+		/// <summary>
+		/// Gets the cropping to use for the given encoding profile and title.
+		/// </summary>
+		/// <param name="profile">The encoding profile to use.</param>
+		/// <param name="title">The title being encoded.</param>
+		/// <returns>The cropping to use for the encode.</returns>
+		private static Cropping GetCropping(EncodingProfile profile, Title title)
+		{
+			Cropping crop;
+			switch (profile.CroppingType)
+			{
+				case CroppingType.Automatic:
+					crop = title.AutoCropDimensions;
+					break;
+				case CroppingType.Custom:
+					crop = profile.Cropping;
+					break;
+				default:
+					crop = new Cropping();
+					break;
+			}
+			return crop;
+		}
 	}
 }
