@@ -12,7 +12,7 @@ namespace HandBrake.Interop.Model.Encoding
 	public enum Detelecine
 	{
 		Off = 0,
-		Default,
-		Custom
+		Default = 2,
+		Custom = 1
 	}
 }
