@@ -9,17 +9,35 @@
 
 namespace HandBrake.Interop.Model
 {
-    using System.ComponentModel.DataAnnotations;
+	using System.ComponentModel.DataAnnotations;
 
-    public enum VideoRangeType
+	/// <summary>
+	/// The video range type.
+	/// </summary>
+	public enum VideoRangeType
 	{
+		/// <summary>
+		/// The entire title.
+		/// </summary>
+		[Display(Name = "All")]
+		All,
+
+		/// <summary>
+		/// A chapter range.
+		/// </summary>
 		[Display(Name = "Chapters")]
-		Chapters,
+		Chapters, 
 
+		/// <summary>
+		/// A timespan range in seconds.
+		/// </summary>
 		[Display(Name = "Seconds")]
-		Seconds,
+		Seconds, 
 
+		/// <summary>
+		/// A frame range.
+		/// </summary>
 		[Display(Name = "Frames")]
 		Frames
 	}
-}
+}
\ No newline at end of file
