@@ -10,6 +10,7 @@
 namespace HandBrake.Interop.SourceData
 {
     using System;
+    using System.Globalization;
 
     /// <summary>
 	/// An object representing a Chapter aosciated with a Title, in a DVD
@@ -22,17 +23,22 @@ namespace HandBrake.Interop.SourceData
 		public int ChapterNumber { get; set; }
 
 		/// <summary>
-		/// Gets or sets the length in time this Chapter spans
+		/// Gets or sets the duration of this chapter.
 		/// </summary>
 		public TimeSpan Duration { get; set; }
 
+		/// <summary>
+		/// Gets or sets the duration of this chapter in PTS.
+		/// </summary>
+		public ulong DurationPts { get; set; }
+
 		/// <summary>
 		/// Override of the ToString method to make this object easier to use in the UI
 		/// </summary>
 		/// <returns>A string formatted as: {chapter #}</returns>
 		public override string ToString()
 		{
-			return this.ChapterNumber.ToString();
+			return this.ChapterNumber.ToString(CultureInfo.InvariantCulture);
 		}
 	}
 }
\ No newline at end of file
